
# http://www.absoft.com/literature/osxuserguide.pdf
# http://www.absoft.com/documentation.html

import os
import sys

from cpuinfo import cpu
from fcompiler import FCompiler, dummy_fortran_file
from misc_util import cyg2win32

class AbsoftFCompiler(FCompiler):

    compiler_type = 'absoft'
    #version_pattern = r'FORTRAN 77 Compiler (?P<version>[^\s*,]*).*?Absoft Corp'
    version_pattern = r'(f90:.*?Absoft Pro FORTRAN Version|FORTRAN 77 Compiler)'+\
                      r' (?P<version>[^\s*,]*)(.*?Absoft Corp|)'

    # samt5735(8)$ f90 -V -c dummy.f
    # f90: Copyright Absoft Corporation 1994-2002; Absoft Pro FORTRAN Version 8.0
    # Note that fink installs g77 as f77, so need to use f90 for detection.

    executables = {
        'version_cmd'  : ["f90", "-V -c %(fname)s.f -o %(fname)s.o" \
                          % {'fname':cyg2win32(dummy_fortran_file())}],
        'compiler_f77' : ["f77"],
        'compiler_fix' : ["f90"],
        'compiler_f90' : ["f90"],
        'linker_so'    : ["f90"],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"]
        }

    if os.name=='nt':
        library_switch = '/out:'      #No space after /out:!

    module_dir_switch = None
    module_include_switch = '-p'

    def get_flags_linker_so(self):
        if os.name=='nt':
            opt = ['/dll']
        else:
            opt = ["-K","shared"]
        return opt

    def library_dir_option(self, dir):
        if os.name=='nt':
            return ['-link','/PATH:"%s"' % (dir)]
        return "-L" + dir

    def library_option(self, lib):
        if os.name=='nt':
            return '%s.lib' % (lib)
        return "-l" + lib

    def get_library_dirs(self):
        opt = FCompiler.get_library_dirs(self)
        d = os.environ.get('ABSOFT')
        if d:
            opt.append(os.path.join(d,'LIB'))
        return opt

    def get_libraries(self):
        opt = FCompiler.get_libraries(self)
        opt.extend(['fio','f90math','fmath'])
        if os.name =='nt':
            opt.append('COMDLG32')
        return opt

    def get_flags(self):
        opt = FCompiler.get_flags(self)
        if os.name != 'nt':
            opt.extend(['-s'])
            if self.get_version():
                if self.get_version()>='8.2':
                    opt.append('-fpic')
        return opt

    def get_flags_f77(self):
        opt = FCompiler.get_flags_f77(self)
        opt.extend(['-N22','-N90','-N110'])
        v = self.get_version()
        if os.name == 'nt':
            if v and v>='8.0':
                opt.extend(['-f','-N15'])
        else:
            opt.append('-f')
            if v:
                if v<='4.6':
                    opt.append('-B108')
                else:
                    # Though -N15 is undocumented, it works with
                    # Absoft 8.0 on Linux
                    opt.append('-N15')
        return opt

    def get_flags_f90(self):
        opt = FCompiler.get_flags_f90(self)
        opt.extend(["-YCFRL=1","-YCOM_NAMES=LCS","-YCOM_PFX","-YEXT_PFX",
                    "-YCOM_SFX=_","-YEXT_SFX=_","-YEXT_NAMES=LCS"])
        if self.get_version():
            if self.get_version()>'4.6':
                opt.extend(["-YDEALLOC=ALL"])                
        return opt

    def get_flags_fix(self):
        opt = FCompiler.get_flags_fix(self)
        opt.extend(["-YCFRL=1","-YCOM_NAMES=LCS","-YCOM_PFX","-YEXT_PFX",
                    "-YCOM_SFX=_","-YEXT_SFX=_","-YEXT_NAMES=LCS"])
        opt.extend(["-f","fixed"])
        return opt

    def get_flags_opt(self):
        opt = ['-O']
        return opt

if __name__ == '__main__':
    from distutils import log
    log.set_verbosity(2)
    from fcompiler import new_fcompiler
    compiler = new_fcompiler(compiler='absoft')
    compiler.customize()
    print compiler.get_version()



from shelve import Shelf

class DbfilenameShelf(Shelf):
	"""Shelf implementation using the "anydbm" generic dbm interface.

	This is initialized with the filename for the dbm database.
	See the module's __doc__ string for an overview of the interface.
	"""
	
	def __init__(self, filename, flag='c'):
		import dumbdbm_patched
		Shelf.__init__(self, dumbdbm_patched.open(filename, flag))


def open(filename, flag='c'):
	"""Open a persistent dictionary for reading and writing.

	Argument is the filename for the dbm database.
	See the module's __doc__ string for an overview of the interface.
	"""
	
	return DbfilenameShelf(filename, flag)



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
from PR import *
from mesh3d import *
from graph3d import *
from plane import *
from GistPlotter import *

def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return

print "Test of slices through an imploding sphere. Type demo () to see the test."
paws ()

def demo () :
   f = PR ('./bills_plot')
   n_nodes = f.NumNodes
   n_z = f.NodesOnZones
   x = f.XNodeCoords
   y = f.YNodeCoords
   z = f.ZNodeCoords
   c = f.ZNodeVelocity
   n_zones = f.NumZones
   # Put vertices in right order for Gist
## n_z = transpose (
##    take (transpose (n_z), array ( [0, 1, 3, 2, 4, 5, 7, 6])))
   
   m1 = Mesh3d (x = x, y = y, z = z, c = c, avs = 1, hex = [n_zones, n_z])
   
   s1 = sslice (m1, .9 * max (c), varno = 1)
   s11 = sslice (m1, -1.8, varno = 1)
## pyz = Plane (array ([1., 0., 0.], Float ), zeros (3, Float))
   pyz = Plane (array ([1., 0., 0.], Float ), array ( [0.0001, 0., 0.], Float))
   pxz = Plane (array ([0., 1., 0.], Float ), array ( [0., 0.0001, 0.], Float))
   p2 = Plane (array ([1., 0., 0.], Float ), array ( [0.35, 0., 0.], Float))
   
   s2 = sslice (m1, pyz, varno = 1, opt_3d = ["wm", "s4"])
   s22 = sslice (m1, p2, varno = 1, opt_3d = ["wm", "s4"])
   s23 = sslice (m1, pxz, varno = 1, opt_3d = ["wm", "s4"])
   
## pl = GistPlotter.Plotter (" ", hcp = "talk.ps", dump = 1, dpi = 75)
## g1 = Graph3d( [s2, s22, s23], plotter = pl, color_card = "rainbowhls",
##      opt_3d = ["wm", "s4"], mask = "min", color_bar = 1, split = 0)
   g1 = Graph3d( [s2, s22, s23], color_card = "rainbowhls",
        opt_3d = ["wm", "s4"], mask = "min", color_bar = 1, split = 0,
        hardcopy = "talk.ps")
   g1.plot ()
   paws ()
   g1.change (split = 1)
   g1.plot ()
   paws ()
   g1.set_surface_list ( [s11, s2, s22, s23])
   g1.plot ()
   paws ()
   s2 = sslice (m1, .9 * min (c), varno = 1, opt_3d = "w3")
   s3 = sslice (m1, .5 * min (c), varno = 1, opt_3d = "w3")
   s4 = sslice (s2, -pyz, opt_3d = "w3")
   s5 = sslice (m1, .5 * max (c), varno = 1, opt_3d = "w3")
   s6 = sslice (s5, -pyz, opt_3d = "w3")
   g1.set_surface_list ( [s1, s2, s6])
   g1.plot ()
   paws ()
   # Do multiple slices
   slice_list = []
   for i in range (8) :
      sl = sslice (m1, .9 * min (c) + i * (.9 * max (c) - .9 * min (c)) / 8., 
         varno = 1, opt_3d = "none")
      slice_list.append (sslice (sl, pxz))
   g1.set_surface_list ( slice_list)
   g1.plot ()
   paws ()




from distutils.command.build_py import build_py as old_build_py

class build_py(old_build_py):

    def find_package_modules(self, package, package_dir):
        modules = old_build_py.find_package_modules(self, package, package_dir)

        # Find build_src generated *.py files.
        build_src = self.get_finalized_command('build_src')
        modules += build_src.py_modules.get(package,[])

        return modules



#!/usr/bin/env python

import os
from glob import glob
from scipy_distutils.misc_util import get_path, default_config_dict, dot_join

def configuration(parent_package='',parent_path=None):
    parent_path2 = parent_path
    parent_path = parent_package
    local_path = get_path(__name__,parent_path2)
    config = default_config_dict('weave',parent_package)
    config['packages'].append(dot_join(parent_package,'weave.tests'))
    test_path = os.path.join(local_path,'tests')
    config['package_dir']['weave.tests'] = test_path
    
    scxx_files = glob(os.path.join(local_path,'scxx','*.*'))
    install_path = os.path.join(parent_path,'weave','scxx')
    config['data_files'].extend( [(install_path,scxx_files)])
    
    blitz_files = glob(os.path.join(local_path,'blitz-20001213','blitz','*.*'))
    install_path = os.path.join(parent_path,'weave','blitz-20001213',
                                'blitz')
    config['data_files'].extend( [(install_path,blitz_files)])
    
    array_files = glob(os.path.join(local_path,'blitz-20001213','blitz',
                                    'array','*.*'))
    install_path = os.path.join(parent_path,'weave','blitz-20001213',
                                'blitz','array')
    config['data_files'].extend( [(install_path,array_files)])
    
    meta_files = glob(os.path.join(local_path,'blitz-20001213','blitz',
                                    'meta','*.*'))
    install_path = os.path.join(parent_path,'weave','blitz-20001213',
                                'blitz','meta')
    config['data_files'].extend( [(install_path,meta_files)])

    swig_files = glob(os.path.join(local_path,'swig','*.c'))
    install_path = os.path.join(parent_path,'weave','swig')
    config['data_files'].extend( [(install_path,swig_files)])

    doc_files = glob(os.path.join(local_path,'doc','*.html'))
    install_path = os.path.join(parent_path,'weave','doc')
    config['data_files'].extend( [(install_path,doc_files)])

    example_files = glob(os.path.join(local_path,'examples','*.py'))
    install_path = os.path.join(parent_path,'weave','examples')
    config['data_files'].extend( [(install_path,example_files)])
    
    return config

if __name__ == '__main__':    
    from scipy_distutils.core import setup
    setup(version = "0.3.0",
          description = "Tools for inlining C/C++ in Python",
          author = "Eric Jones",
          author_email = "eric@enthought.com",
          licence = "SciPy License (BSD Style)",
          url = 'http://www.scipy.org',
          **configuration(parent_path=''))



import common_info
from c_spec import common_base_converter
import sys,os

# these may need user configuration.
if sys.platform == "win32":
    wx_base = r'c:\third\wxpython-2.4.0.7'
else:
    # probably should do some more discovery here.
    wx_base = '/usr/lib/wxPython'

def get_wxconfig(flag):
    wxconfig = os.path.join(wx_base,'bin','wx-config')
    import commands
    res,settings = commands.getstatusoutput(wxconfig + ' --' + flag)
    if res:
        msg = wxconfig + ' failed. Impossible to learn wxPython settings'
        raise RuntimeError, msg
    return settings.split()

wx_to_c_template = \
"""
class %(type_name)s_handler
{
public:    
    %(c_type)s convert_to_%(type_name)s(PyObject* py_obj, const char* name)
    {
        %(c_type)s wx_ptr;        
        // work on this error reporting...
        if (SWIG_GetPtrObj(py_obj,(void **) &wx_ptr,"_%(type_name)s_p"))
            handle_conversion_error(py_obj,"%(type_name)s", name);
        %(inc_ref_count)s
        return wx_ptr;
    }
    
    %(c_type)s py_to_%(type_name)s(PyObject* py_obj,const char* name)
    {
        %(c_type)s wx_ptr;        
        // work on this error reporting...
        if (SWIG_GetPtrObj(py_obj,(void **) &wx_ptr,"_%(type_name)s_p"))
            handle_bad_type(py_obj,"%(type_name)s", name);
        %(inc_ref_count)s
        return wx_ptr;
    }    
};

%(type_name)s_handler x__%(type_name)s_handler = %(type_name)s_handler();
#define convert_to_%(type_name)s(py_obj,name) \\
        x__%(type_name)s_handler.convert_to_%(type_name)s(py_obj,name)
#define py_to_%(type_name)s(py_obj,name) \\
        x__%(type_name)s_handler.py_to_%(type_name)s(py_obj,name)

"""

class wx_converter(common_base_converter):
    def __init__(self,class_name="undefined"):
        self.class_name = class_name
        common_base_converter.__init__(self)

    def init_info(self):
        common_base_converter.init_info(self)
        # These are generated on the fly instead of defined at 
        # the class level.
        self.type_name = self.class_name
        self.c_type = self.class_name + "*"
        self.return_type = self.class_name + "*"
        self.to_c_return = None # not used
        self.check_func = None # not used
        self.headers.append('"wx/wx.h"')
        if sys.platform == "win32":        
            # These will be used in many cases
            self.headers.append('<windows.h>')        
            
            # These are needed for linking.
            self.libraries.extend(['kernel32','user32','gdi32','comdlg32',
                                   'winspool', 'winmm', 'shell32', 
                                   'oldnames', 'comctl32', 'ctl3d32',
                                   'odbc32', 'ole32', 'oleaut32', 
                                   'uuid', 'rpcrt4', 'advapi32', 'wsock32'])
                                   
            # not sure which of these macros are needed.
            self.define_macros.append(('WIN32', '1'))
            self.define_macros.append(('__WIN32__', '1'))
            self.define_macros.append(('_WINDOWS', '1'))
            self.define_macros.append(('STRICT', '1'))
            # I think this will only work on NT/2000/XP set
            # set to 0x0400 for earlier versions.
            # Hmmm.  setting this breaks stuff
            #self.define_macros.append(('WINVER', '0x0350'))

            self.library_dirs.append(os.path.join(wx_base,'lib'))
            #self.include_dirs.append(os.path.join(wx_base,'include'))            
            self.include_dirs.append(wx_base)            
            self.include_dirs.append(os.path.join(wx_base,'include'))            
            self.include_dirs.append(os.path.join(wx_base,'include','msw'))            
            # how do I discover unicode or not unicode??            
            # non-unicode            
            self.libraries.append('wxmsw24h')
            self.include_dirs.append(os.path.join(wx_base,'lib'))
            
            # unicode
            #self.libraries.append('wxmswuh')
            #self.include_dirs.append(os.path.join(wx_base,'lib','mswdlluh'))
            #self.define_macros.append(('UNICODE', '1'))
        else:
            # make sure the gtk files are available 
            # ?? Do I need to link to them?
            self.headers.append('"gdk/gdk.h"')
            # !! This shouldn't be hard coded.
            self.include_dirs.append("/usr/include/gtk-1.2")
            self.include_dirs.append("/usr/include/glib-1.2")
            self.include_dirs.append("/usr/lib/glib/include")
            cxxflags = get_wxconfig('cxxflags')
            libflags = get_wxconfig('libs') + get_wxconfig('gl-libs')
            
            #older versions of wx do not support the ldflags.
            try:
                ldflags = get_wxconfig('ldflags')
            except RuntimeError:
                ldflags = []
                    
            self.extra_compile_args.extend(cxxflags)
            self.extra_link_args.extend(libflags)
            self.extra_link_args.extend(ldflags)            
        self.support_code.append(common_info.swig_support_code)
    
    def type_match(self,value):
        is_match = 0
        try:
            wx_class = value.this.split('_')[-2]
            if wx_class[:2] == 'wx':
                is_match = 1
        except AttributeError:
            pass
        return is_match

    def generate_build_info(self):
        if self.class_name != "undefined":
            res = common_base_converter.generate_build_info(self)
        else:
            # if there isn't a class_name, we don't want the
            # we don't want the support_code to be included
            import base_info
            res = base_info.base_info()
        return res
        
    def py_to_c_code(self):
        return wx_to_c_template % self.template_vars()

    #def c_to_py_code(self):
    #    return simple_c_to_py_template % self.template_vars()
                    
    def type_spec(self,name,value):
        # factory
        class_name = value.this.split('_')[-2]
        new_spec = self.__class__(class_name)
        new_spec.name = name        
        return new_spec

    def __cmp__(self,other):
        #only works for equal
        res = -1
        try:
            res = cmp(self.name,other.name) or \
                  cmp(self.__class__, other.__class__) or \
                  cmp(self.class_name, other.class_name) or \
                  cmp(self.type_name,other.type_name)
        except:
            pass
        return res



#
# linalg - Linear algebra routines
#

from info_linalg import __doc__
from linalg_version import linalg_version as __version__

from basic import *
from decomp import *
from matfuncs import *
from blas import *
from iterative import *




""" Vector Quantization Module

    Provides several routines used in creating a code book from a set of
    observations and comparing a set of observations to a code book.
    
    All routines expect an "observation vector" to be stored in each
    row of the obs matrix.  Similarly the codes are stored row wise
    in the code book matrix.

    whiten(obs) -- 
        Normalize a group of observations on a per feature basis
    vq(obs,code_book) -- 
        Calculate code book membership of obs
    kmeans(obs,k_or_guess,iter=20,thresh=1e-5) -- 
        Train a codebook for mimimum distortion using the kmeans algorithm
    
"""
from Numeric import *
from scipy_base.fastumath import *
import scipy
from scipy.stats import randint
from scipy_base import common_type as _common_type

def whiten(obs):
    """ Normalize a group of observations on a per feature basis

        Description
        
            Before running kmeans algorithms, it is beneficial to "whiten", or
            scale, the observation data on a per feature basis.  This is done
            by dividing each feature by its standard deviation across all 
            observations.
            
        Arguments
        
            obs -- 2D array.
                    Each row of the array is an observation.  The
                    columns are the "features" seen during each observation

                              #   f0    f1    f2
                        obs = [[  1.,   1.,   1.],  #o0
                               [  2.,   2.,   2.],  #o1
                               [  3.,   3.,   3.],  #o2
                               [  4.,   4.,   4.]]) #o3

            XXX perhaps should have an axis variable here.
            
        Outputs
        
            result -- 2D array.
                    Contains the values in obs scaled by the standard devation 
                    of each column.

        Test
        
            >>> features  = array([[  1.9,2.3,1.7],
            ...                    [  1.5,2.5,2.2],
            ...                    [  0.8,0.6,1.7,]])
            >>> whiten(features)
            array([[ 3.41250074,  2.20300046,  5.88897275],
                   [ 2.69407953,  2.39456571,  7.62102355],
                   [ 1.43684242,  0.57469577,  5.88897275]])

    """
    std_dev = scipy.std(obs,axis=0)
    return obs / std_dev

def vq(obs,code_book):
    """ Vector Quantization: assign features sets to codes in a code book.

        Description:
            Vector quantization determines which code in the code book best
            represents an observation of a target.  The features of each
            observation are compared to each code in the book, and assigned 
            the one closest to it.  The observations are contained in the obs 
            array. These features should be "whitened," or nomalized by the 
            standard deviation of all the features before being quantized.  
            The code book can be created using the kmeans algorithm or 
            something similar.
    
        Note: 
           This currently forces 32 bit math precision for speed.  Anyone know
           of a situation where this undermines the accuracy of the algorithm?
            
            
        Arguments:
            obs -- 2D array.
                    Each row of the array is an observation.  The
                    columns are the "features" seen during each observation
                    The features must be whitened first using the
                    whiten function or something equivalent.
            code_book -- 2D array.
                    The code book is usually generated using the kmeans
                    algorithm.  Each row of the array holds a different
                    code, and the columns are the features of the code.
                                    #   c0    c1    c2   c3
                        code_book = [[  1.,   2.,   3.,   4.],  #f0
                                     [  1.,   2.,   3.,   4.],  #f1
                                     [  1.,   2.,   3.,   4.]]) #f2
        Outputs:
            code -- 1D array.
                    If obs is a NxM array, then a length M array
                    is returned that holds the selected code book index for
                    each observation.
            dist -- 1D array.
            		The distortion (distance) between the observation and
            		its nearest code        
        Reference
        
        Test
        
            >>> code_book = array([[1.,1.,1.],
            ...                    [2.,2.,2.]])
            >>> features  = array([[  1.9,2.3,1.7],
            ...                    [  1.5,2.5,2.2],
            ...                    [  0.8,0.6,1.7]])
            >>> vq(features,code_book)
            (array([1, 1, 0],'i'), array([ 0.43588989,  0.73484692,  0.83066239]))

    """
    try:
        import _vq
        from scipy.misc import x_common_type
        ct = x_common_type(obs,code_book)
        c_obs = obs.astype(ct)
        c_code_book = code_book.astype(ct)
        if   ct == 'f':
            results = _vq.float_vq(c_obs,c_code_book)
        elif ct == 'd': 
            results = _vq.double_vq(c_obs,c_code_book)
        else:
            results = py_vq(obs,code_book)
    except ImportError:
        results = py_vq(obs,code_book)
    return results        

def py_vq(obs,code_book):
    """ Python version of vq algorithm.
    
        This function is slower than the C versions, but it works for 
        all input types.  If the inputs have the wrong types for the 
        C versions of the function, this one is called as a last resort.
        
        Its about 20 times slower than the C versions.
    """
    No,Nf = shape(obs) #No = observation count, Nf = feature count
    # code books and observations should have same number of features
    assert(Nf == code_book.shape[1])
    code = [];min_dist = []    
    #create a memory block to use for the difference calculations    
    diff = zeros(shape(code_book),_common_type(obs,code_book))
    for o in obs:
        subtract(code_book,o,diff) # faster version of --> diff = code_book - o
        dist = sqrt(sum(diff*diff,-1))
        code.append(argmin(dist))
        #something weird here dst does not work reliably because it sometime
        #returns an array of goofy length. Try except fixes it, but is ugly.
        dst = minimum.reduce(dist,0)
        try:    dst = dst[0]
        except: pass
        min_dist.append(dst) 
    return array(code,typecode=Int), array(min_dist)

def py_vq2(obs,code_book):
    """ This could be faster when number of codebooks is small, but it becomes
         a real memory hog when codebook is large.  It requires NxMxO storage
         where N=number of obs, M = number of features, and O = number of 
         codes.
    """
    No,Nf = shape(obs) #No = observation count, Nf = feature count
    # code books and observations should have same number of features
    assert(Nf == code_book.shape[1])
    diff = obs[NewAxis,:,:]-code_book[:,NewAxis,:]
    dist = sqrt(sum(diff*diff,-1))
    code = argmin(dist,0)
    min_dist = minimum.reduce(dist,0) #the next line I think is equivalent 
                                      #  - and should be faster
    #min_dist = choose(code,dist) # but in practice, didn't seem to make 
                                  # much difference.
    return code, min_dist
       
def kmeans_(obs,guess,thresh=1e-5):
    """ See kmeans
    
    Outputs
    
        code_book -- the lowest distortion codebook found.
        avg_dist -- the average distance a observation is
                    from a code in the book.  Lower means
                    the code_book matches the data better.
    Test
    
        Note: not whitened in this example.
        
        >>> features  = array([[ 1.9,2.3],
        ...                    [ 1.5,2.5],
        ...                    [ 0.8,0.6],
        ...                    [ 0.4,1.8],
        ...                    [ 1.0,1.0]])        
        >>> book = array((features[0],features[2]))
        >>> kmeans_(features,book)        
        (array([[ 1.7       ,  2.4       ],
               [ 0.73333333,  1.13333333]]), 0.40563916697728591)

    """
    
    code_book = array(guess,copy=1)
    Nc = code_book.shape[0]
    avg_dist=[]
    diff = thresh+1.
    while diff>thresh:
        #compute membership and distances between obs and code_book
        obs_code, distort = vq(obs,code_book)
        avg_dist.append(scipy.mean(distort,axis=-1))
        #recalc code_book as centroids of associated obs
        if(diff > thresh):
            has_members = []
            for i in arange(Nc):                
                cell_members = compress(equal(obs_code,i),obs,0)
                if cell_members.shape[0] > 0:
                    code_book[i] = scipy.mean(cell_members,0)
                    has_members.append(i)
            #remove code_books that didn't have any members
            code_book = take(code_book,has_members,0)
        if len(avg_dist) > 1:
            diff = avg_dist[-2] - avg_dist[-1]
    #print avg_dist
    return code_book, avg_dist[-1]

def kmeans(obs,k_or_guess,iter=20,thresh=1e-5): 
    """ Generate a code book with minimum distortion
        
    Description
            
    Arguments
    
        obs -- 2D array
                Each row of the array is an observation.  The
                columns are the "features" seen during each observation
                The features must be whitened first using the 
                whiten function or something equivalent.                                             
        k_or_guess -- integer or 2D array.
            If integer, it is the number of code book elements.
            If a 2D array, the array is used as the intial guess for
            the code book.  The array should have k rows, and the
            same number of columns (features) as the obs array.
        iter -- integer.
            The number of times to restart the kmeans algorithm with
            a new initial guess.  If k_or_guess is a 2D array (codebook),
            this argument is ignored and only 1 iteration is run.           
        thresh -- float
            Terminate each kmeans run when the distortion change from 
            one iteration to the next is less than this value.  
    Outputs
    
        codesbook -- 2D array.
            The codes that best fit the observation
        distortion -- float
            The distortion between the observations and the codes.                
            
    Reference
    
    Test
    
        ("Not checked carefully for accuracy..." he said sheepishly)
        
        >>> features  = array([[ 1.9,2.3],
        ...                    [ 1.5,2.5],
        ...                    [ 0.8,0.6],
        ...                    [ 0.4,1.8],
        ...                    [ 0.1,0.1],
        ...                    [ 0.2,1.8],
        ...                    [ 2.0,0.5],
        ...                    [ 0.3,1.5],
        ...                    [ 1.0,1.0]])        
        >>> whitened = whiten(features)
        >>> book = array((whitened[0],whitened[2]))
        >>> kmeans(whitened,book)        
        (array([[ 2.3110306 ,  2.86287398],
               [ 0.93218041,  1.24398691]]), 0.85684700941625547)
               
        >>> import RandomArray       
        >>> RandomArray.seed(1000,2000)
        >>> codes = 3
        >>> kmeans(whitened,codes)
        (array([[ 2.3110306 ,  2.86287398],
               [ 1.32544402,  0.65607529],
               [ 0.40782893,  2.02786907]]), 0.5196582527686241)
               
    """
    if int(iter) < 1:
        raise ValueError, 'iter must be >= to 1.'
    if type(k_or_guess) == type(array([])): 
        guess = k_or_guess
        result = kmeans_(obs,guess,thresh=thresh)
    else:
        best_dist = 100000 #initialize best distance value to a large value
        No = obs.shape[0]
        k = k_or_guess
        #print 'kmeans iter: ',
        for i in range(iter):   
            #print i,
            #the intial code book is randomly selected from observations
            guess = take(obs,randint(0,No,k),0) 
            book,dist = kmeans_(obs,guess,thresh=thresh)
            if dist < best_dist: 
                best_book = book
                best_dist = dist
        #print      
        result = best_book,best_dist
    return result               




## License for the Python wrapper
## ==============================

## Copyright (c) 2004 David M. Cooke <cookedm@physics.mcmaster.ca>

## Permission is hereby granted, free of charge, to any person obtaining a copy of
## this software and associated documentation files (the "Software"), to deal in
## the Software without restriction, including without limitation the rights to
## use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
## of the Software, and to permit persons to whom the Software is furnished to do
## so, subject to the following conditions:

## The above copyright notice and this permission notice shall be included in all
## copies or substantial portions of the Software.

## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
## SOFTWARE.

## Modifications by Travis Oliphant and Enthought, Inc.  for inclusion in SciPy

import scipy_base as NA
import _lbfgsb
import optimize

approx_fprime = optimize.approx_fprime

def fmin_l_bfgs_b(func, x0, fprime=None, args=(),
                  approx_grad = False,
                  bounds=None, m=10, factr=1e7, pgtol=1e-5,
                  epsilon=1e-8,
                  iprint=-1, maxfun=15000):
    """
    Minimize a function func using the L-BFGS-B algorithm.

    Arguments:

    func    -- function to minimize. Called as func(x, *args)

    x0      -- initial guess to minimum

    fprime  -- gradient of func. If None, then func returns the function
               value and the gradient ( f, g = func(x, *args) ).
               Called as fprime(x, *args)

    args    -- arguments to pass to function

    approx_grad -- if true, approximate the gradient numerically    

    bounds  -- a list of (min, max) pairs for each element in x, defining
               the bounds on that parameter. Use None for one of min or max
               when there is no bound in that direction

    m       -- the maximum number of variable metric corrections
               used to define the limited memory matrix. (the limited memory BFGS
               method does not store the full hessian but uses this many terms in an
               approximation to it).

    factr   -- The iteration stops when
               (f^k - f^{k+1})/max{|f^k|,|f^{k+1}|,1} <= factr*epsmch

               where epsmch is the machine precision, which is automatically
               generated by the code. Typical values for factr: 1e12 for
               low accuracy; 1e7 for moderate accuracy; 10.0 for extremely
               high accuracy.

    pgtol   -- The iteration will stop when
                  max{|proj g_i | i = 1, ..., n} <= pgtol
               where pg_i is the ith component of the projected gradient.   

    epsilon -- step size used when approx_grad is true, for numerically
               calculating the gradient

    iprint  -- controls the frequency of output. <0 means no output.

    maxfun  -- maximum number of function evaluations.


    Returns:
    x, f, d = fmin_lbfgs_b(func, x0, ...)
    
    x -- position of the minimum
    f -- value of func at the minimum
    d -- dictionary of information from routine
        d['warnflag'] is
            0 if converged,
            1 if too many function evaluations,
            2 if stopped for another reason, given in d['task']
        d['grad'] is the gradient at the minimum (should be 0 ish)
        d['funcalls'] is the number of function calls made.


   License of L-BFGS-B (Fortran code)
   ==================================

   The version included here (in fortran code) is 2.1 (released in 1997). It was
   written by Ciyou Zhu, Richard Byrd, and Jorge Nocedal <nocedal@ece.nwu.edu>. It
   carries the following condition for use:

   This software is freely available, but we expect that all publications
   describing  work using this software , or all commercial products using it,
   quote at least one of the references given below.

   References
     * R. H. Byrd, P. Lu and J. Nocedal. A Limited Memory Algorithm for Bound
       Constrained Optimization, (1995), SIAM Journal on Scientific and
       Statistical Computing , 16, 5, pp. 1190-1208.
     * C. Zhu, R. H. Byrd and J. Nocedal. L-BFGS-B: Algorithm 778: L-BFGS-B,
       FORTRAN routines for large scale bound constrained optimization (1997),
       ACM Transactions on Mathematical Software, Vol 23, Num. 4, pp. 550 - 560.
    """
    n = len(x0)

    if bounds is None:
        bounds = [(None,None)] * n
    if len(bounds) != n:
        raise ValueError('length of x0 != length of bounds')

    if approx_grad:
        def func_and_grad(x):
            f = func(x, *args)
            g = approx_fprime(x, func, epsilon, *args)
            return f, g
    elif fprime is None:
        def func_and_grad(x):
            f, g = func(x, *args)
            return f, g
    else:
        def func_and_grad(x):
            f = func(x, *args)
            g = fprime(x, *args)
            return f, g

    nbd = NA.zeros((n,), NA.Int)
    low_bnd = NA.zeros((n,), NA.Float)
    upper_bnd = NA.zeros((n,), NA.Float)
    bounds_map = {(None, None): 0,
              (1, None) : 1,
              (1, 1) : 2,
              (None, 1) : 3}
    for i in range(0, n):
        l,u = bounds[i]
        if l is not None:
            low_bnd[i] = l
            l = 1
        if u is not None:
            upper_bnd[i] = u
            u = 1
        nbd[i] = bounds_map[l, u]

    x = NA.array(x0)
    f = NA.array(0.0, NA.Float64)
    g = NA.zeros((n,), NA.Float64)
    wa = NA.zeros((2*m*n+4*n + 12*m**2 + 12*m,), NA.Float64)
    iwa = NA.zeros((3*n,), NA.Int)
    task = NA.zeros((60,), NA.Character)
    csave = NA.zeros((60,), NA.Character)
    lsave = NA.zeros((4,), NA.Int)
    isave = NA.zeros((44,), NA.Int)
    dsave = NA.zeros((29,), NA.Float64)

    task[:] = 'START'

    n_function_evals = 0
    while 1:
#        x, f, g, wa, iwa, task, csave, lsave, isave, dsave = \
        _lbfgsb.setulb(m, x, low_bnd, upper_bnd, nbd, f, g, factr,
                       pgtol, wa, iwa, task, iprint, csave, lsave,
                       isave, dsave)
        task_str = task.tostring()
        if task_str.startswith('FG'):
            # minimization routine wants f and g at the current x
            n_function_evals += 1
            f[0], g = func_and_grad(x, *args)
        elif task_str.startswith('NEW_X'):
            # new iteration
            if n_function_evals > maxfun:
                task[:] = 'STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT'
        else:
            break

    task_str = task.tostring().strip()
    if task_str.startswith('CONV'):
        warnflag = 0
    elif n_function_evals > maxfun:
        warnflag = 1
    else:
        warnflag = 2


    d = {'grad' : g,
         'task' : task_str,
         'funcalls' : n_function_evals,
         'warnflag' : warnflag
        }
    return x, f[0], d

if __name__ == '__main__':
    def func(x):
        f = 0.25*(x[0]-1)**2
        for i in range(1, x.shape[0]):
            f += (x[i] - x[i-1]**2)**2
        f *= 4
        return f
    def grad(x):
        g = NA.zeros(x.shape, NA.Float)
        t1 = x[1] - x[0]**2
        g[0] = 2*(x[0]-1) - 16*x[0]*t1
        for i in range(1, g.shape[0]-1):
            t2 = t1
            t1 = x[i+1] - x[i]**2
            g[i] = 8*t2 - 16*x[i]*t1
        g[-1] = 8*t1
        return g
    
    factr = 1e7
    pgtol = 1e-5

    n=25
    m=10

    bounds = [(None,None)] * n
    for i in range(0, n, 2):
        bounds[i] = (1.0, 100)
    for i in range(1, n, 2):
        bounds[i] = (-100, 100)

    x0 = NA.zeros((n,), NA.Float)
    x0[:] = 3

    x, f, d = fmin_l_bfgs_b(func, x0, fprime=grad, m=m,
                            factr=factr, pgtol=pgtol)
    print x
    print f
    print d
    x, f, d = fmin_l_bfgs_b(func, x0, approx_grad=True,
                            m=m, factr=factr, pgtol=pgtol)
    print x
    print f
    print d



#!/usr/bin/env python

# This python script tests the numpyio module.
# also check out numpyio.fread.__doc__ and other method docstrings.

import os,sys
import unittest
from unittest import TestCase
from scipy_test.testing import *
set_package_path()
import io
from io import numpyio
del sys.path[0]


import Numeric
N = Numeric
import tempfile

class test_numpyio(TestCase):
    def check_basic(self):
        # Generate some data
        a = 255*rand(20)
        # Open a file
        fname = tempfile.mktemp('.dat')
        fid = open(fname,"wb")
        # Write the data as shorts
        numpyio.fwrite(fid,20,a,Numeric.Int16)
        fid.close()
        # Reopen the file and read in data
        fid = open(fname,"rb")
        print "\nDon't worry about a warning regarding the number of bytes read."
        b = numpyio.fread(fid,1000000,Numeric.Int16,Numeric.Int)
        fid.close()
        assert(N.product(a.astype(N.Int16) == b))
        os.remove(fname)

class test_read_array(TestCase):
    def check_complex(self):
        a = rand(13,4) + 1j*rand(13,4)
        fname = tempfile.mktemp('.dat')
        io.write_array(fname,a)
        b = io.read_array(fname,atype=N.Complex)
        assert_array_almost_equal(a,b,decimal=4)
        os.remove(fname)

    def check_float(self):
        a = rand(3,4)*30
        fname = tempfile.mktemp('.dat')
        io.write_array(fname,a)
        b = io.read_array(fname)
        assert_array_almost_equal(a,b,decimal=4)
        os.remove(fname)

    def check_integer(self):
        from scipy import stats
        a = stats.randint.rvs(1,20,size=(3,4))
        fname = tempfile.mktemp('.dat')
        io.write_array(fname,a)
        b = io.read_array(fname,atype=N.Int)
        assert_array_equal(a,b)
        os.remove(fname)

if __name__ == "__main__":
    ScipyTest('io.array_import').run()





""" Cow --- Cluster Of Workstations.
    Description

        cow is a package that includes tools for interacting with a cluster
        of workstations as a single computational engine.  It is well suited
        for "embarassingly parallel" problems that fit comfortably into a 
        master-slave paradigm.  For such problems, it is often possible to
        convert them from parallel to serial execution by changing a few lines
        of code.  Parallel Genetic Algorithms or Monte Carlo simulations are 
        two classes of problems that can benefit significantly from using
        cow.  If your problem requires closely coupled parallelism, take a 
        look at pyMPI.

        cow also includes a reasonably complete set of tools administering a 
        cluster of machines from Python such as process control, gathering
        system information and executing shell commands on remote processes
        simultaneously.
            
        cow should graze happily in heterogeneous cluster environments, though
        it hasn't tested extensively there.
        
        See scipy.cow.machine_cluster for more information and examples.
"""

import sync_cluster, socket
import time
import os # for getuid()

ClusterError = 'ClusterError'
TimeoutError = 'TimeoutError'
      
class machine_cluster:
    """ Treats a cluster of workstations as a single computational engine.
    
        Description
        
            machine_cluster simplifies interacting with a cluster of 
            workstations.  machine_cluster starts a group of slave
            interpreters.  Setting and retreiving global variables on these
            machines is accomplished through its dictionary interface.  
            You can also call functions or execute code blocks on the remote
            machines.  A couple of routines provide automatic parallelization
            of looping constructs.  It also provides routines for cluster
            wide process control similar to ps and cluster wide machine
            information on load, CPU, and memory information.
            
            
        Caveats
        
            Some functionality is only available on Linux. Unix is better
            supported in general than MSWindows.  See user manual for details.
        
            Cow assumes you have ssh access to all slave machines.  If not,
            the start() method will fail and you'll need to start all slaves
            by hand using::
            
                python sync_cluster.py server <port>
    
        Example Usage::
        
            # start two slave interpreters on the local machine at the
            # specified ports
            >>> slave_list = [ ('127.0.0.1',10000), ('127.0.0.1',10001)]
            >>> cluster = scipy.cow.machine_cluster(slave_list)
            >>> cluster.start()
            # set and retreive a global variable on the slave interpreters
            >>> cluster['a'] = 1 
            >>> cluster['a']
            (1, 1)
            >>> import string
            # process a list of strings in parallel converting them to
            # upper case (illustrative example only -- dismal performance)
            >>> string_list = ['aaa','bbb','ccc','ddd']
            >>> cluster.loop_apply(string.upper,0,(string_list,))
            ['AAA','BBB','CCC','DDD']
            >>> cluster.info()
            MACHINE   CPU        GHZ   MB TOTAL  MB FREE   LOAD
            bull      2xP3       0.5     960.0     930.0   0.00
            bull      2xP3       0.5     960.0     930.0   0.00
    """
    def __init__(self,server_list):        
        """ machine_cluster(slave_list) --> cluster_object
        
            Description
            
                Create a cluster object from a list of slave machines.
                slave_list is a list of 2-tuples of the form (address, port)
                that specify the network address and port where the slave
                interpreters will live and listen.  The address is always
                a string and can be either the machine name or its IP address.
                The port should be an unused port on the slave machine.
                Always use a port number higher than 1024.
            
            Example::
                
                # example 1 using IP addresses
                >>> slave_list = [ ('127.0.0.1',10000), ('127.0.0.1',10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)                
                # example 2 using machine names                
                >>> slave_list = [ ('node0',11500), ('node1',11500)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
        """
        self.workers=[]
        self.worker_by_name={}
        worker_id = 1
        for host,port in server_list:
            # Add the uid here can help with port conflicts, but only works
            # on Unix clusters.  We really need to work out a daemon service
            # model that makes the port mess transparent.
            port = port #+ os.getuid()
            new_worker = sync_cluster.standard_sync_client(host,port,worker_id)
            self.workers.append(new_worker)
            self.worker_by_name[host] = new_worker
            worker_id = worker_id + 1
                                   
    def start(self,force_restart=0,timeout=60):
        """ start(force_restart=0, timeout=60) -- Start remote slave processes.
        
            Description
            
                Start the remote slave interpreters in the cluster.  The timeout
                value is specified in seconds and defaults to 60.  The timeout
                starts counting down only after ssh/rsh has tried to start
                all the remote processes.  This means the actual time in the
                function could be much longer than 60 seconds - depending
                on how long rsh/ssh takes.
                
                Its possible the 60 second time out will be to short for large
                clusters - but I hope not!
                
            Caveats                
                
                start() is not supported on MSWindows because of the lack of
                standard/robust support for remote startup and background 
                processing in the CMD shell.
        """    
        if not force_restart and self.is_running():
            return
        # start the worker processes.
        for worker in self.workers:
            worker.start_server()

        if not self.is_running():
            print '                      Starting Servers'
            print ' |----|----|----15---|----|----30---|----|----45---' \
                   '|----|----60'
            print '0.',
            stop_watch = timer()
            stop_watch.start()
            minute = 0
        import sys    
        while not self.is_running():
            if stop_watch.current_lap() > 1:
                sys.stdout.write('.')
                stop_watch.mark_lap()               
            elapsed = stop_watch.elapsed()
            if (elapsed - minute * 60) > 60:
                minute = minute + 1
                print
                print minute,
            if elapsed > timeout:
                raise TimeoutError  
        print 'servers running!'
            
    def stop(self):
        """ stop() -- Tell all remote slaves to terminate.
        
            Description
            
                stop calls sys.exit(0) on all the slave processes so that 
                they will terminate gracefully.  Note that if, for some
                reason, you are unable to connect to a remote processes due
                to some socket error, you'll have to kill the slave process
                by hand.
        """
        for worker in self.workers:
            import sys; sys.stdout.flush()
            try: worker.exec_code('import sys;sys.exit(0)')
            except:
                #should really do something here to
                # trap non-SystemExit errors.  
                pass
    def restart(self):
        """ restart() -- terminate all remote slaves and restart them.
        
            restart is useful when you would like to reset all slave
            interpreters to a known state.
        """
        self.stop()
        self.start()

    def is_running(self,timeout=0):
        """ is_running(timeout=0) --> 0 or 1
        
            check all the slave processes in the cluster are up and running.
            if timeout is specified, is_running will continually check if the 
            cluster is_running until it either gets a positive result or gives
            up and returns 0 after the specified number of seconds.
        """
        
        # wait for them to start           
        import time
        st = time.time()            
        still_waiting = 1
        while still_waiting: 
            try:
                # Send a simple command to all workers
                # and wait till they handle it successfully
                self.exec_code("1==1") 
            except ClusterError:
                still_waiting = 1
                elapsed = time.time() - st
                if elapsed > timeout:
                    # We've run out of time.
                    return 0
            else:
                still_waiting = 0    
            wait_time = time.time() - st                                
        # should we somehow dessiminate worker topology (ids)
        # to all machines here?
        return 1
        
    def _send(self,package,addendum=None):
        """ _send(package, addendum=None) -- send a package to all slaves.
        
            Description
            
                _send takes a package packed up by a packer object (see
                sync_cluster) and sends it to each of the slave processes.
                addendum is either None or a list with the same length as
                there are slave processes.  Each entry is a small package
                of additional information that is to be sent to a specific
                slave process.  It contains data that is only needed by that
                process.
                
            Implementation Notes
            
                The send is done synchronously to each worker in turn.  The
                entire package is sent to slave0 before moving on and sending
                the message to slave1.
                
                If a socket error occurs while trying to send data to a
                given slave, the offending worker is pushed into the 
                self.has_send_error list.  Also, self.send_exc is a dictionary
                that stores the (err_type,err_msg) as the key and the offending
                worker as the value.  This information is used in recv to 
                skip receiving from slaves who failed on send and also for 
                error reporting.
        """        
        if addendum: 
            N = len(addendum)
            assert(N <= len(self.workers))
        else:
            N = len(self.workers)            
        self.send_exc = {}
        self.had_send_error = []
        for i in range(N):
            try:
                if not addendum:
                    self.workers[i].send(package)
                else:
                    self.workers[i].send(package,addendum[i])
            except socket.error, msg:    
                import sys
                err_type, err_msg = str,sys.exc_info()[:2]
                self.had_send_error.append(self.workers[i])
                key = (err_type,err_msg)
                try:
                    self.send_exc[key].append(self.workers[i].id)
                except: 
                    self.send_exc[key] = [self.workers[i].id]
            # else - handle other errors?               
        self.Nsent = N
                    
    def _recv(self):
        """ _recv() -- retreive results from slave processes.
        
            Description
            
                Retreive results from all slave processes that were 
                successfully sent a package.  If an error occurs while
                receiving from one of the slaves, the error is noted and
                the results from the other slaves are retreived.  A tuple
                the results from all workers is returned by _recv.  An
                entry of None is placed in the tuple for any worker that
                had an error.      
                
                The recv is done synchronously, waiting for slave0 to return
                its results before moving on to slave1 to recv its results.          
        """

        self.had_recv_error = []
        self.recv_exc = {}        
        results = []
        import sys; 
        #only listen on workers involved in calculation.
        for worker in self.workers[:self.Nsent]:
            if worker in self.had_send_error:
                results.append(None)
            else:    
                try:
                    sys.stdout.flush()
                    results.append(worker.recv())
                except sync_cluster.RemoteError:    
                    import sys
                    err = sys.exc_info()[1]
                    # Force the err msg (err[1]) to be a string.
                    # This dimishes info content, but makes sure
                    # that the sames errors are hashed correctly
                    # in the dictionary. (does it?)   
                    err_type,err_msg, err_traceback = err
                    err = err_type,str(err_msg), err_traceback
                    self.had_recv_error.append(worker)                    
                    try: self.recv_exc[err].append(worker.id)
                    except: self.recv_exc[err] = [worker.id]
                    results.append(None)
                except sync_cluster.RemoteCrashError:
                    # Gotta be more intelligent here...
                    msg =  'Error! Remote worker %d appears to have crashed.' \
                            % worker.id
                    raise sync_cluster.RemoteCrashError,msg           
                # else handle other errors    
        #print                
        return tuple(results)                
    
    def _send_recv(self,package,addendum=None):
        """ _send_recv(package,addendum=None) --> results
        
            Description
            
                send a message to each worker in turn and then immediately
                began listening for the results.  All sends are done before
                listening for results from any of the slave processes.  See
                _send and _recv for more information.
                
                If an error occurs during either the send or recv phases,
                the handle_error() method is called.  If know errors are found,
                a tuple containing the results from each slave is returned.
                
                If an error does occur and an exception is raised, it is still
                possible to retreive the set of results that executed correctly
                from the last_results attribute.
        """        
        self._send(package,addendum)
        self.last_results = self._recv()
        if(len(self.send_exc) or len(self.recv_exc)):
            self.handle_error()
        return self.last_results    

    def handle_error(self):
        """ handle_error() -- make sense of send and recv errors
        
            Description
            
                Error handling attempts to examine the errors that occuer
                during remote execution and report them in the least verbose
                manner.  If the same error occurs on all slaves, it tries to
                only report it once.  Otherwise it reports all the errors
                that occur on slaves and prints the slaves traceback.
                
                Currently error handling is pretty simplistic.  It'd be nice
                if socket errors were viewed as severe and the slave either
                restarted or marked as dead and its work distributed among
                the other workers.
                
        """
        # perhaps do some nifty stuff here to 
        # mark bad workers, try to restart, etc.
        msg = ''
        Nworkers = len(self.workers)
        Nsend_errors = len(self.had_send_error)
        Nsend_error_types = len(self.send_exc.keys())
        Nrecv_errors = len(self.had_recv_error)
        Nrecv_error_types = len(self.recv_exc.keys())        
        if (Nsend_errors == Nworkers and
            Nsend_error_types == 1):    
            sock_err_type,err_msg = self.send_exc.keys()[0]
            if sock_err_type == 111:
                # An attempt at helpful info for a common problem.
                msg =       '\n\nConnection refused on all workers.\n'
                msg = msg + '    Perhaps restarting the cluster would help.\n'
                msg = msg + '    Use Your_Clusters_Name_Here.restart()'
            else:
                msg =       'A Socket error occured sending to all workers.\n\t'
                msg = msg + str(sock_err_type) + ': ' + str(err_msg)                
        elif Nsend_errors:
            msg = '\n\nThe following errors occured when sending data:\n\t'
            for err,guilty_workers in self.send_exc.items():                
                msg = msg + str(err) + '\n\t'
                msg = msg + 'Guilty workers: ' + str(guilty_workers) + '\n'    
        
        if (Nrecv_errors == Nworkers and
              Nrecv_error_types == 1):
                err,dummy = self.recv_exc.items()[0]                    
                err_type, err_msg, err_traceback = err
                msg =       '\n\nThe same error occured on all workers:\n\t'
                msg = msg + str(err_type) + ': ' + str(err_msg)                
                msg = msg + err_traceback
        elif Nrecv_errors:
            msg = '\n\nThe following errors occured on workers:\n\t'
            for err,guilty_workers in self.recv_exc.items():                
                err_type, err_msg, err_traceback = err
                msg = msg + str(err_type) + ': ' + str(err_msg) + '\n'              
                msg = msg + 'Guilty workers: ' + str(guilty_workers) + '\n'    
                msg = msg + err_traceback
                

        raise ClusterError, msg
        
    ##############################################################
    # slave processor info
    ##############################################################
    def load(self):
        """ load() -- print human readable load information for slave hosts 
            
            Description
            
                The load value printed is the 1 minute load average that
                is commonly printed by uptime on Unix machines.
                
                load depends on the implementation of scipy_proc on each
                slave's host OS. It will not work for Windows slave processes.
                However, if you are using a Windows master to control a Linux
                cluster of slaves, it should work fine.                

            Example::

                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> cluster.load()
                    n0: 0.00, n1: 0.00
        """        
        import string
        import scipy_distutils.proc as scipy_proc
        results = self.load_list()
        for i in range(len(self.workers)):            
            name = string.split(self.workers[i].host,'.')[0]
            res = results[i]
            s = "%6s: %1.2f," % (name[-6:], res['load_1'])
            print s,
            if not ((i+1) % 5):
                print
    
    def info(self):
        """ info() -- print human readable info about the slave hosts 
            
            Description
            
                Print out the each slave interpreters host name, number
                and type of processors, memory usage, and current load 
                information in human readable form.
                
                info depends on the implementation of scipy_proc on each
                slave's host OS. It will not work for Windows slave processes.
                However, if you are using a Windows master to control a Linux
                cluster of slaves, it should work fine.                

            Example::

                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> cluster.info()
                MACHINE   CPU        GHZ   MB TOTAL  MB FREE   LOAD
                n0        2xP3       0.4     192.0      13.0   0.00
                n1        2xP3       0.4     192.0      22.0   0.00
        """                
        import string
        results = self.info_list()
        labels = "%-8s  %-9s  %-4s  %-8s  %-8s  %-4s" % \
                 ('MACHINE','CPU','GHZ','MB TOTAL', 
                  'MB FREE','LOAD')
        print labels          
        for i in range(len(self.workers)):            
            name = string.split(self.workers[i].host,'.')[0]
            res = results[i]
            s = "%-8s %2dx%-6s  %4.1f  %8.1f  %8.1f   %4.2f" %  \
                (name[-8:], res['cpu_count'],res['cpu_type'][-6:], \
                 res['cpu_speed'],res['mem_total'],res['mem_free'],\
                 res['load_1'])
            print s

    def load_list(self):        
        """ load_list() -- Return a list of slave load information dictionaries
            
            Description
            
                Retreive a dictionary with information about the load on each
                host processor.  The dictionaries have three keys, load1, 
                load5, and load15 indicating the 1, 5, and 15 minute load 
                averages for the processor.  These could be useful for (as 
                yet unimplemented) load balancing schemes.

                load_list depends on the implementation of scipy_proc on each
                slave's host OS. It will not work for Windows slave processes.
                However, if you are using a Windows master to control a Linux
                cluster of slaves, it should work fine.                

            Example::

                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> cluster.load_info()
                ({'load_5': 0.0, 'load_1': 0.0, 'load_15': 0.0},
                 {'load_5': 0.0, 'load_1': 0.0, 'load_15': 0.0})
        """        
        import scipy_distutils.proc as scipy_proc
        res = self.apply(scipy_proc.load_avg,())
        return res

    def info_list(self):
        """ info() -- print human readable info about the slave hosts 
            
            Description
            
                Print out the each slave interpreters host name, number
                and type of processors, memory usage, and current load 
                information in human readable form.
                
                info depends on the implementation of scipy_proc on each
                slave's host OS. It will not work for Windows slave processes.
                However, if you are using a Windows master to control a Linux
                cluster of slaves, it should work fine.                

            Example::

                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> cluster.info()
                MACHINE   CPU        GHZ   MB TOTAL  MB FREE   LOAD
                n0        2xP3       0.4     192.0      13.0   0.00
                n1        2xP3       0.4     192.0      22.0   0.00
    
        """                
        import scipy_distutils.proc as scipy_proc
        res = self.apply(scipy_proc.machine_info,())
        return res

    ##############################################################
    # slave process information and control
    ##############################################################
    def ps(self,sort_by='cpu',**filters):
        """ ps(sort_by='cpu',**filters) -- list processes on slave machines.
        
            Description
                
                List all the processes on all remote slave machines.  This
                is like a cluster-wide Unix ps command and is output in a
                similar human readable form.  The sort_by argument allows
                you to sore the process list by various fields including,
                pid, cpu, user, machine, memory, state and command.  keyword
                arguments are used as filters to limit the number of processes
                displayed.  For example, the keyword, user='ej' will only list
                processes for user ej and cpu='>10' will only list processes 
                using more th 50% of the cpu cycles.
                
                ps depends on the implementation of scipy_proc on each
                slave's host OS. It will not work for Windows slave processes.
                However, if you are using a Windows master to control a Linux
                cluster of slaves, it should work fine.                

            Example::

                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> cluster.ps(user='ej')
                MACHINE USER       PID  %CPU  %MEM TOTAL MB   ...
                n0     ej        22915  99.9   2.1    4.027   ...
                n0     ej        22916  99.9   2.1    4.055   ...
                n1     ej        22915  99.9   2.1    4.027   ...
                n1     ej        22916  99.9   2.1    4.055   ...
                ...
                
        """
        psl = self.ps_list(sort_by,**filters)
        if len(psl):
            print psl[0].labels_with_name()
        for i in psl: print i.str_with_name()
    
    def ps_list(self,sort_by='cpu',**filters):
        """ ps_list(self,sort_by='cpu',**filters) -- get cluster processes
        
            Description
            
                Return a list containing one scipy_proc.process objects for 
                each process running on the cluster host machines.  process 
                objects contain a ton of information about cpu, memory, etc.
                used by the process.
                
                See ps for more information.
            
            Example:
                        
                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> p = cluster.ps_list()
                >>> for i in p: print p.pid
                ...
                22890 22889 22889 22890 1071 1071 ...
        """
        import operator
        import scipy_distutils.proc as scipy_proc
        res = self.apply(scipy_proc.ps_list,())
        psl = reduce(operator.add,res)
        psl = scipy_proc.ps_sort(psl,sort_by,**filters)        
        return psl
 
    def nice(self,increment=10):
        """ nice(increment=10) --> success_list
        
            increment all slave interpreter's nice value by increment.            
            hmmm. this doesn't seem to work. see os.nice()
        """
        res = self.apply(os.nice,(increment,))
        return res

    def renice(self,process_list,level):
        """ renice(process_list,level) -- set nice value multiple processes 

            Description
            
                Change the nice level of multiple remote processes. 
                process_list is a list of scipy_proc.process objects. 
                level is the new nice value for the listed processes.
             
            Caveats

                Once niced down, a process cannot be reniced back up.
                This is a Linux issue.
        """    
        res = []
        pids = {}
        for process in process_list:
            if hasattr(process,'machine'):
                try:
                    worker = self.worker_by_name[process.machine] 
                except KeyError:
                    worker = self.worker_by_name[process.long_machine] 
                pid = process.pid
            else:
                worker = self.workers[process[0]] 
                pid = process[1]
            try:
                pids[worker] = pids[worker] + ' ' + str(pid)
            except:
                pids[worker] = str(pid)
        for worker,value in pids.items():
            arg = 'renice %d -p %s' % (level,value)
            res.append(worker.apply(os.system,(arg,)))
        return res

    def kill(self,process_list,signal = 'TERM'):
        """ kill(self,process_list,signal = 'TERM') -- Signal process list.

            Description
            
                Send a signal to all of the scipy_proc.process objects in
                the process_list.  This is usually used to kill the processes.
                The signal may be given as a signal name or number.
        """    
        res = []
        pids = {}
        for process in process_list:
            if hasattr(process,'machine'):
                try:
                    worker = self.worker_by_name[process.machine] 
                except KeyError:
                    worker = self.worker_by_name[process.long_machine] 
                pid = process.pid
            else:
                worker = self.workers[process[0]] 
                pid = process[1]
            try:
                pids[worker] = pids[worker] + ' ' + str(pid)
            except:
                pids[worker] = str(pid)
        for worker,value in pids.items():
            arg = 'kill -s ' + signal + ' %s' % (level,value)
            res.append(worker.apply(os.system,(arg,)))
        return res
        
    def system(self,cmd):
        """ system(cmd) -- execute cmd on all remote machines

            A list of all the remote responses is returned.  Unlike
            os.system which returns the exit value of the cmd string,
            this function returns the text output by the command.
        """            
        code = 'import os;f=os.popen("%s");res = f.read(-1);f.close();' % cmd
        return self.exec_code(code,returns=['res'])

    def reload(self,module):
        """ reload(module) -- reload module on all remote interpreters
        
            module can either be the name of a module or the actual
            module object.
        """        
        try: 
            code = 'import %s; reload(%s)' % ((module.__name__,)*2)
        except AttributeError:
            code = 'import %s; reload(%s)' % ((module,)*2)
        self.workers.exec_code(code)    
        
    ##############################################################
    # remote code and function execution
    ##############################################################
    
    # mirror all of sync_client functions
    # They assumes all clients have the same packing procedures.
    def exec_code(self,code,inputs=None,returns=None):
        """ exec_code(code,inputs=None,returns=None)
        
            Similar to Python's exec statement. Execute the same code fragment
            on all remote interpreter. inputs is a dictionary of variable 
            values to use when executing the code. returns is a list of
            variable names that should be returned after executing the code.  
            If one value is specified, the value for that variable is returned.
            If multiple values are specified, a tuple is returned.
            
            exec_code returns a list of the values requested variables,
            one entry for each slave.
        """
        #use the first worker to package up the cmd.
        package = self.workers[0].exec_code_pack(code,inputs,returns)
        return self._send_recv(package)

    def apply(self,function,args=(),keywords=None):        
        """ apply(function,args=(),keywords=None)
        
            Similar to Python's builtin apply method.  Execute the given
            function with the argument list, args, and keyword arguments,
            keywords, on each of the slave processes.
            
            apply returns a list of the results from calling function,
            one result for each slave.
        """        
        package = self.workers[0].apply_pack(function,args,keywords)
        return self._send_recv(package)

    def loop_apply(self,function,loop_var,args=(),keywords=None):
        """ loop_apply(function,loop_var, args=(),keywords=None)
        
            Description

                Call function with the given args and keywords.  One of the
                arguments or keywords is actually a sequence of arguments.  
                This sequence is looped over, calling function once for each
                value in the sequence. loop_var indicates which variable to
                loop over.  If an integer, loop_var indexes the args list.
                If a string, it specifies a keyword variable.  The loop sequence
                is divided as evenly as possible between the worker nodes and
                executed in parallel.

            Example::
                
                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> import string
                >>> string_list = ['aaa','bbb','ccc','ddd']
                >>> cluster.loop_apply(string.upper,0,(string_list,))
                ['AAA','BBB','CCC','DDD']
            
        """                
        #----------------------------------------------------
        # Prepare main package for sending
        # almost verbatim from loop_apply_pack in sync_cluster
        #----------------------------------------------------
        #if type(loop_var) == type(1):
        #    loop_var = function.func_code.co_varnames[loop_var]
        all_keywords = {}
        if keywords: all_keywords.update(keywords)
        #more_keywords = sync_cluster.args_to_keywords(function,args)
        #sync_cluster.catch_keyword_conflicts(more_keywords,all_keywords)
        #all_keywords.update(more_keywords)

        # pull out the loop variable.
        if type(loop_var) != type(''):
            loop_var = int(loop_var)
            loop_data = args[loop_var]
            # no need to pack and send since it'll be in the "addendum"
            args = list(args)
            args[loop_var] = None
            args = tuple(args)
        else:
            loop_data = all_keywords[loop_var]    
            # no need to pack and send since it'll be in the "addendum"
            del all_keywords[loop_var]             
        contents={'_command':sync_cluster.loop_func,'function':function,
                  'args':args,'keywords':all_keywords,'loop_var':loop_var}
        package = self.workers[0].packer.pack(contents) 
        return self.loop_send_recv(package,loop_data,loop_var)

    def loop_code(self,code,loop_var,inputs=None,returns=None):
        """ loop_code(code,loop_var,inputs=None,returns=None)
        
            Description
        
                Similar to exec_code and loop_apply.  Here loop_var indicates 
                the variable name in the inputs dictionary that is looped over.
            
            Example::
                
                >>> slave_list = [('n0',10000), ('n1', 10001)]
                >>> cluster = scipy.cow.machine_cluster(slave_list)
                >>> cluster.start()
                >>> import string
                >>> a = [1, 2, 3, 4]
                >>> cluster.loop_code("b=a*2",'a',{'a':a},('b',))
                (2, 4, 6, 8)
        """
        the_inputs = {}
        the_inputs.update(inputs)    
        loop_data = the_inputs[loop_var]
        the_inputs[loop_var] = None #make it small for packing
        package = self.workers[0].loop_code_pack(code,loop_var,
                                                 the_inputs,returns)
        return self.loop_send_recv(package,loop_data,loop_var)
    
    # array specific routines
    def row_split(self,name,sequence):
        """experimental"""
        import scipy
        q=scipy.split(sequence,len(self.workers))
        self.loop_code(name+'=_q_','_q_',inputs={'_q_':q},returns=(),
                        global_vars=(name,))
    def row_gather(self,name):
        """experimental"""
        from Numeric import concatenate
        return concatenate(self[name])
        
    def loop_send_recv(self,package,loop_data,loop_var):        
        #----------------------------------------------------
        # Now split the loop data evenly among the workers,
        # pack them up as addendums to the original package,
        # and send them off for processing.
        #----------------------------------------------------
        job_groups = equal_balance(loop_data,len(self.workers))
        addendums = []
        for grp in job_groups:
            addendums.append({loop_var:grp})
        results = self._send_recv(package,addendums)
        # Nothing done here to figure out the output format.
        # It is always returned as a tuple
        # Probably will be handier to have it as a 
        # Numeric array sometimes.
        out = ()
        for result_group in results:
            out = out + result_group
        return out

    ##############################################################
    # dictionary interface
    ##############################################################

    def __getitem__(self, key):
        # currently allowing tuples also!
        #assert(type(key) is type(''))
        package = self.workers[0].get_pack(key)    
        return self._send_recv(package)

    def __setitem__(self, key, item):
        assert(type(key) is type(''))
        package = self.workers[0].update_pack({key:item})    
        return self._send_recv(package)

    def __delitem__(self, key):
        # currently allowing tuples also!
        # assert(type(key) is type(''))
        package = self.workers[0].del_pack(key)    
        return self._send_recv(package)
        
    def update(self, dict):
        package = self.workers[0].update_pack(dict)    
        return self._send_recv(package)

def equal_balance(jobs,Nworkers):
    """ Split jobs into Nworkers equal groups.    
        When an equal split is not possible,
        the larger groups occur at the front
        of the list.
    """
    
    #no jobs to do - return empty group list.
    if not len(jobs): return ()
    Ntotal_jobs = len(jobs)
    
    # find the number of jobs each wroker must do
    # for everyone to have equal work loads
    group_size = Ntotal_jobs / Nworkers
    
    # if there are jobs left over, some of the workers
    # will need to do 1 extra job.
    if Ntotal_jobs % Nworkers:
        group_size = group_size + 1

    # after some algebra, we can solve for the
    # number, a, of workers that need to do extra work
    a = Ntotal_jobs + Nworkers - Nworkers*group_size                       
    if a*group_size < Ntotal_jobs:
        b = Nworkers - a
    else:
        b = 0        
    
    # a workers do an extra job, b workers do standard
    # number of jobs.        
    group_sizes = a*[group_size] + b*[group_size-1]
    
    # now split the jobs up into groups for each of
    # the workers.
    last = 0
    job_groups = []
    for size in group_sizes:
        next = last+size
        job_groups.append(jobs[last:next])
        last = next
#    sum = 0        
#    for grp in job_groups:
#        sum = sum + len(grp)        
#    assert(sum,Ntotal_jobs)           
    return tuple(job_groups)

import operator
class timer:

    def __init__(self):
        self.reset()
    def reset(self):
        self.total_t = 0
        self.lap_start = 0
        self.running = 0
        self.lap_list = [0]        
    def start(self):
        if not self.running:
            self.running = 1
            self.start_t = time.time()                
        else:
            print 'already running: use reset() to start the timer back at 0.'
                
    def stop(self):
        self.running = 0
        elapsed_t = time.time() - self.start_t
        self.total_t = self.total_t + elapsed_t
        self.lap_list[-1] = self.lap_list[-1] + elapsed_t
        return self.total_t
    def elapsed(self):
        if self.running:
            return self.total_t + (time.time() - self.start_t)
        else:
            return self.total_t
    def mark_lap(self):
        self.lap_start = self.elapsed()
        self.lap_list.append(self.lap_start-self.lap_list[-1])
    def get_laps(self):
        return self.lap_list[1:] +[self.elapsed()-self.lap_list[-1]]               
    def current_lap(self):
            return self.elapsed()-self.lap_start
        
                

if __name__ == '__main__':
    borg = cluster(server_list)    
    borg.start()



import unittest
from Numeric import *
# The following try/except so that non-SciPy users can still use blitz
try:
    from scipy_base.fastumath import *
except:
    pass # scipy_base.fastumath not available    
import RandomArray
import time

from scipy_test.testing import *
set_package_path()
from weave import standard_array_spec
restore_path()

def remove_whitespace(in_str):
    import string
    out = string.replace(in_str," ","")
    out = string.replace(out,"\t","")
    out = string.replace(out,"\n","")
    return out
    
def print_assert_equal(test_string,actual,desired):
    """this should probably be in scipy_test.testing
    """
    import pprint
    try:
        assert(actual == desired)
    except AssertionError:
        import cStringIO
        msg = cStringIO.StringIO()
        msg.write(test_string)
        msg.write(' failed\nACTUAL: \n')
        pprint.pprint(actual,msg)
        msg.write('DESIRED: \n')
        pprint.pprint(desired,msg)
        raise AssertionError, msg.getvalue()

class test_array_converter(unittest.TestCase):    
    def check_type_match_string(self):
        s = standard_array_spec.array_converter()
        assert( not s.type_match('string') )
    def check_type_match_int(self):
        s = standard_array_spec.array_converter()        
        assert(not s.type_match(5))
    def check_type_match_array(self):
        s = standard_array_spec.array_converter()        
        assert(s.type_match(arange(4)))

if __name__ == "__main__":
    ScipyTest('weave.standard_array_spec').run()




# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from slice3 import plane3
from Numeric import *
from scipy_base.fastumath import *
from graftypes import *

class Plane :

   """Plane (normal, point) simply embodies a plane through the
   given point with the given normal.
   """

   def type (self) :
       return PlaneType

   def __init__ (self, normal = array ([1., 0., 0.]),
                 point = array ([0., 0., 0.])) :
       self.coeffs = plane3 (normal, point)

   def __repr__ (self) :
       return `self.coeffs`

   def __str__ (self) :
       return `self.coeffs`

   def __neg__ (self) :
       p = Plane ()
       p.coeffs = - self.coeffs
       return p

   def rep (self) :
       return self.coeffs

   def astype (self, newtype) :
       return (array (self.coeffs, newtype))



#!/usr/bin/env python

from scipy_distutils.misc_util import default_config_dict

def configuration(parent_package='',parent_path=None):
    package = 'cow'
    config = default_config_dict(package,parent_package)
    return config

if __name__ == '__main__':
    from scipy_distutils.core import setup
    setup(**configuration())



"""
    This module is separated from gui_thread because it imports
    wxPython.  We can't do that at the top level of gui_thread.
    It must be done in the wxPython thread.  Classes that need
    wxPython classes in there definitions (inheritance) are 
    banished to this module.
    
    If someone comes up with a way to dump this into gui_thread
    that isn't ugly, we may consider doing that.  I'm sure
    there is a way.  It's the 'isn't ugly' part that concerns me...
    
    The file has the following contents:
    
    o An event handler that is chained onto any proxified window 
      to notifies the wrapping proxy object when the window has 
      been closed.
    
    o Proxy Event class used to pass requests from the main thread
      to the wxPython thread
    
    o A base class for all proxy objects called proxy_base

    o A proxied_callable class that handles callable attributes of a
      proxied clas by making the call secondary thread.

    o A proxy_attr class that handles attributes of a proxied object
      correctly by dispatching calls to the secondary thread.
    
    o A wxPython class called event_catcher that lives in the
      secondary thread and catches/dispatches all events requested
      from the primary thread
    
    o The main wxApp that lives in the secondary thread (very simple)
      
"""
import sys, os
if sys.platform != 'win32':
    # For some reason, PythonWin hangs indefinitely with this code.
    _in_thread = 0
    if not sys.modules.has_key('wxPython'):
        _in_thread = 1
        print '<Importing wxPython... ',
        sys.stdout.flush()
    from wxPython.wx import *
    if _in_thread:
        print 'done.>\n>>>',
        sys.stdout.flush()
    del _in_thread
else:
    print_import = 0 
    if not sys.modules.has_key('wxPython'):
        print_import = 1
    from wxPython.wx import *
    if print_import:
        print "<wxPython imported>\n>>>",

import thread, threading
import types, traceback
import UserList, UserDict
import main

#################################
# Window Close Event Handler 
#################################        

class CloseEvtHandler(wxEvtHandler):
    """ An Event Handler added to all proxified objects to notify
        the proxy wrapper when the object has been closed.    
    """
    def __init__(self,proxy_obj):
        """ Build an event handler to handle the Close event.
            It needs access to the proxy object to tell it when
            the underlying wxPython window has been closed.
        """
        wxEvtHandler.__init__(self)
        self.proxy_obj = proxy_obj
        EVT_CLOSE(self,self.ProxyOnCloseWindow)
    def ProxyOnCloseWindow(self,evt):        
        """ Tell the proxy wrapper that the object is no longer
             alive.  Logic in the wrapper methods use this info
             to make sure they don't call a dead wxPython object.   
        """
        try:
            self.proxy_obj.kill_proxy()
        except AttributeError:
            self.proxy_obj.proxy_object_alive = 0
        del self.proxy_obj
        evt.Skip()

#################################
# Proxy Event Object
#################################        

wxEVT_PROXY_EVENT = 26001
def PROXY_EVENT(win, func):
    win.Connect(-1, -1, wxEVT_PROXY_EVENT, func)    
        
class proxy_event(wxPyEvent):
    """ Proxy events transport a method code
        object, the arguments to this method
        and a synchronization method from one
        thread to another.
    """
    def __init__(self,method,args,kw,finished):    
        """ Create a proxy_event object.
        
            method -- a code object that is to be called
            args   -- list of arguments.  if the call is
                      to an unbound method, the first entry
                      should be a class object of the 
                      appropriate type.
            kw     -- dictionary of key word arguments.
            finished -- a threading.Event() object used
                        for syncrhonization. 
        """
        wxPyEvent.__init__(self)
        self.SetEventType(wxEVT_PROXY_EVENT)
        self.method = method
        self.args = args
        self.kw = kw            
        self.finished = finished

	
#################################
# Very useful utility functions.
#################################    

def is_immutable(x):
    """ Checks if object is completely immutable.  A tuple is not
    considered completely immutable since it could contain references
    to objects that could change.  Returns 1 if it is immutable or 0
    if object is mutable."""    
    imm = ()
    try:
        imm = (types.StringType, types.FloatType, types.IntType,
               types.ComplexType, types.NoneType, types.UnicodeType)
    except AttributeError:
        imm = (types.StringType, types.FloatType, types.IntType,
               types.ComplexType, type.NoneType)
    if type(x) in imm:
        return 1
    else:
        return 0

def is_numeric_array(x):
    try:
        x.typecode() in ['c','b','l','d','f','D','F']
        return 1
    except AttributeError:
        pass
    return 0
    
def smart_return(ret):
    
    """ This intelligently returns an appropriately proxied object to
    prevent crashing the interpreter.  If the object is immutable it
    simply returns it.  If it is callable it returns a
    proxied_callable if it is a list/tuple/dictionary it returns a
    special proxied list or dictionary, and if it is none of these it
    returns a proxy_attr."""
    
    if callable(ret):
        return proxied_callable(ret)
    elif is_immutable(ret):
        return ret
    elif type(ret) in (types.ListType, types.TupleType):
        return proxy_list(ret)
    elif type(ret) is types.DictType:
        return proxy_dict(ret)
    elif is_numeric_array(ret):
        return ret
    else:
        return proxy_attr(ret)

    
def smart_call(func, *args, **kw):    
    """Performs a function call using the passed function/bound method
    by dispatching the method to the secondary thread when necessary.
    If the execution is already in the secondary thread then no
    proxying is done.
    
      func -- A callable function, a bound method or function.

      args -- Arguments to be passed to the function.

      kw -- Keyword arguments.
    """
    ret_val = None
    d_args = main.dereference_arglist(args)
    d_kw = main.dereference_dict(kw)
    if main.in_proxy_call:
	ret_val = apply(func, d_args, d_kw)
    else:
	finished = threading.Event()
	evt = proxy_event(func, d_args, d_kw, finished)
	wxPostEvent(proxy_base.catcher, evt)
	finished.wait()
	if finished.exception_info:
	    raise finished.exception_info[0], \
	          finished.exception_info[1]
	ret_val = finished._result
    return smart_return(ret_val)


#################################
# A proxied callable object
#################################

class proxied_callable:

    """This wraps any callable object so that the call is made via a
    the threaded event_catcher using a proxy_event.  This makes it
    possible for the user to call methods of a proxy's attribute.  """

    def __init__(self, call_obj):        
        """ Create the proxied callable object.

            call_obj -- the callable attribute that is being proxied.
        """
        self.__dont_mess_with_me_unless_you_know_what_youre_doing = call_obj
        try:
            call_obj.__doc__
        except AttributeError:
            pass
        else:
            self.__doc__ = call_obj.__doc__
            
    def __call__(self, *args, **kw):        
        """Performs the call to the proxied callable object by
        dispatching the method to the secondary thread."""
        obj = self.__dont_mess_with_me_unless_you_know_what_youre_doing
	my_args = list(args)
	my_args.insert(0, obj)
	return apply(smart_call, my_args, kw)


################################################################
# A proxied list that proxies lists and tuples safely and yet
# behaves very much like them.
################################################################

class proxy_list(UserList.UserList):
    def __init__(self, val):        
        # Do NOT use the UserList __init__ since it makes a copy of
        # the data
        if type(val) is types.TupleType:
            self.data = list(val)
        else:
            self.data = val

    def __getitem__(self, i):
        val = self.data[i]
        return smart_return(val)

    def __getslice__(self, i, j):
        i = max(i, 0); j = max(j, 0)
        return self.__class__(self.data[i:j])


################################################################
# A proxied list that proxies dictionaries safely and yet behaves
# very much like one.
################################################################

class proxy_dict(UserDict.UserDict):
    def __init__(self, val):
        # Do NOT use the UserDict __init__ since it makes a copy of
        # the data
        self.data = val

    def __getitem__(self, key):
        val = self.data[key]
        return smart_return(val)

    def keys(self):
        return smart_return(self.data.keys())

    def items(self):
        return smart_return(self.data.items())

    def values(self):
        return smart_return(self.data.values())

    def get(self, key, failobj=None):
        val = self.data.get(key, failobj)
        return smart_return(val)

    def setdefault(self, key, failobj=None):
        if not self.data.has_key(key):
            self.data[key] = failobj
        return smart_return(self.data[key])

    def popitem(self):
        return smart_return(self.data.popitem())


################################################################
# A proxied attribute that handles callable internal attrubutes
# properly.
################################################################

class proxy_attr:

    """
    Description

      This wraps a proxy's attribute such that a call to the
      attribute's member function returns a proxied_callable and any
      request for a data attribute will return another proxy_attr.
      This way a user can call a method of a proxy's attribute.  Use
      the get_members and get_methods member functions to get a list
      of the data and functions of the proxied object.

    Caveats

      The user can always use
      _proxy_attr__dont_mess_with_me_unless_you_know_what_youre_doing
      and still get around proxying but atleast they were warned. :)
      Also, this isnt a normal Python object in that
      self.__class__.__dict__ will not contain any methods of the
      proxied object.  Only self.__dict__ will contain these.  It
      works but isn't perfect.

    """
    __DONT_WRAP = ('__del__', '__init__', '__getattr__', '__setattr__')

    def __init__(self, obj):
        """ Create the proxied attribute.

            obj -- the attribute that is being proxied.
        """
        self.__dict__['_proxy_attr__dont_mess_with_me_unless_you_know_what_youre_doing'] = obj
        if hasattr(obj, '__class__'):
            try:
                doc = obj.__class__.__doc__
                self.__dict__['__doc__'] = doc
            except AttributeError:
                pass
            self._set_attrs(obj, main.get_all_methods(obj.__class__))
        else:
            # this is most probably a built in type but in any case
            self._set_attrs(obj, dir(obj))

    def _get_meth (self, obj, name):
        if name in self.__DONT_WRAP:
            return None
        try:
            ret = getattr(obj, name)
        except AttributeError:
            return None
        if callable(ret):
            return proxied_callable(ret)
        elif is_immutable(ret):
            return ret
        else:
            return None

    def _set_attrs(self, obj, names):
        for i in names:
            tmp = self._get_meth(obj, i)
            if tmp:
                self.__dict__[i] = tmp

    def get_members(self):
        """Returns a list of the members of the proxied attribute."""
        obj = self.__dont_mess_with_me_unless_you_know_what_youre_doing
        return dir(obj)

    def get_methods(self):
        """Returns a list of the methods of the proxied attribute."""
        obj = self.__dont_mess_with_me_unless_you_know_what_youre_doing
        try:
            return dir(obj.__class__)
        except AttributeError:
            return dir(obj)

    def __getattr__(self, key):
        obj = self.__dont_mess_with_me_unless_you_know_what_youre_doing
        ret = getattr(obj, key)
        return smart_return(ret)

    def __setattr__(self, key, val):
        obj = self.__dont_mess_with_me_unless_you_know_what_youre_doing
	return apply(smart_call, (setattr, obj, key, val))


#################################
# Base class for all automatically generated proxy classes
#################################        

class proxy_base:
    is_proxy = 1
    # all proxy objects share the same event catcher.
    # This is set to be a single instance of the
    # event_catcher class when the second thread is 
    # started.
    catcher = None
    
    def kill_proxy(self):
        """ Used by the CloseEvtHandler to notify
            proxy class that underlying wxPython window
            has been closed.
        """
        self.proxy_object_alive = 0
        
    def post(self,evt):
        """ All proxy classes send their events
            to a single "catcher" window using
            this method.  The catcher is set at 
            startup in the OnInit method of 
            second_thread_app.
        """
        wxPostEvent(self.catcher, evt)        

    def __getattr__(self,key):        
        try:
            ret = self.__dict__[key]
        except KeyError:
            ret = getattr(self.__dict__['wx_obj'],key)
        return smart_return(ret)
    
    def __setattr__(self,key,val):
        if self.__dict__.has_key(key):
            self.__dict__[key] = val
        else:        
            obj = self.wx_obj
            return apply(smart_call, (setattr, obj, key, val))


#################################
# Receives/Dispatches all events requested from the main thread.
#################################        

class event_catcher(wxFrame):
    """ The "catcher" frame in the second thread.
        It is invisible.  It's only job is to receive
        proxy_events from the main thread, and then call
        the method indicated in the event.
    """ 
    def __init__(self):
        wxFrame.__init__(self, NULL,-1,'',size=(300,300))        
        EVT_CLOSE(self,self.OnCloseWindow)
        PROXY_EVENT(self,self.proxy_handler)    
    def proxy_handler(self,evt): 
        try:
            main.in_proxy_call = 1
            evt.finished._result = apply(evt.method,evt.args,evt.kw)
            evt.finished.exception_info = None
            main.in_proxy_call = 0
            evt.finished.set()
        except:
            type, value, tb = sys.exc_info()
            info = traceback.extract_tb(tb)
            filename, lineno, function, text = info[-1] # last line only
            tmp = "\nFile \"%(filename)s\", line %(lineno)s, "\
                  "in %(function)s\n%(text)s"%locals()
            value = str(value) + tmp
            evt.finished.exception_info = [type, value]
            type = value = tb = None # clean up
            main.in_proxy_call = 0            
            evt.finished.set()

    def OnCloseWindow(self,evt):
        main.app.ExitMainLoop()


#################################
# The (very simple) wxApp that lives in secondary thread
#################################        
                                
class second_thread_app(wxApp):
    """  wxApp that lives in the second thread.
    """    
    def OnInit(self):
        """ Registers the event_catcher window as 
            the object to whom proxy classes should 
            send their events.
        """
        catcher = event_catcher()
        proxy_base.catcher = catcher
        return true




# Functions which need the PIL

from scipy_base import ppimport
import types
import Numeric

from scipy_base import exp, amin, amax, ravel, asarray, cast, arange, \
     ones, NewAxis, transpose, mgrid, iscomplexobj, sum, zeros

Image = ppimport('Image')
ImageFilter = ppimport('ImageFilter')

__all__ = ['fromimage','toimage','imsave','imread','bytescale',
           'imrotate','imresize','imshow','imfilter','radon']

_UInt8 = Numeric.UnsignedInt8

# Returns a byte-scaled image
def bytescale(data, cmin=None, cmax=None, high=255, low=0):
    if data.typecode == _UInt8:
        return data
    high = high - low
    if cmin is None:
        cmin = amin(ravel(data))
    if cmax is None:
        cmax = amax(ravel(data))
    scale = high *1.0 / (cmax-cmin or 1)
    bytedata = ((data*1.0-cmin)*scale + 0.4999).astype(_UInt8)
    return bytedata + cast[_UInt8](low)
            
def imread(name,flatten=0):
    """Read an image file from a filename.

    Optional arguments:

     - flatten (0): if true, the image is flattened by calling convert('F') on
     the resulting image object.  This flattens the color layers into a single
     grayscale layer.
    """

    im = Image.open(name)
    return fromimage(im,flatten=flatten)

def imsave(name, arr):
    """Save an array to an image file.
    """
    im = toimage(arr)
    im.save(name)
    return

def fromimage(im, flatten=0):
    """Takes a PIL image and returns a copy of the image in a Numeric container.
    If the image is RGB returns a 3-dimensional array:  arr[:,:,n] is each channel

    Optional arguments:

    - flatten (0): if true, the image is flattened by calling convert('F') on
    the image object before extracting the numerical data.  This flattens the
    color layers into a single grayscale layer.  Note that the supplied image
    object is NOT modified.
    """
    assert Image.isImageType(im), "Not a PIL image."
    if flatten:
        im = im.convert('F')
    mode = im.mode
    adjust = 0
    if mode == '1':
        im = im.convert(mode='L')
        mode = 'L'
        adjust = 1
    str = im.tostring()
    type = 'b'
    if mode == 'F':
        type = 'f'
    if mode == 'I':
        type = 'i'
    arr = Numeric.fromstring(str,type)
    shape = list(im.size)
    shape.reverse()
    if mode == 'P':
        arr.shape = shape
        if im.palette.rawmode != 'RGB':
            print "Warning: Image has invalid palette."
            return arr
        pal = Numeric.fromstring(im.palette.data,type)
        N = len(pal)
        pal.shape = (int(N/3.0),3)
        return arr, pal
    if mode in ['RGB','YCbCr']:
        shape += [3]
    elif mode in ['CMYK','RGBA']:
        shape += [4]
    arr.shape = shape
    if adjust:
        arr = (arr != 0)
    return arr

_errstr = "Mode is unknown or incompatible with input array shape."
def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
            mode=None,channel_axis=None):
    """Takes a Numeric array and returns a PIL image.  The mode of the
    PIL image depends on the array shape, the pal keyword, and the mode
    keyword.

    For 2-D arrays, if pal is a valid (N,3) byte-array giving the RGB values
    (from 0 to 255) then mode='P', otherwise mode='L', unless mode is given
    as 'F' or 'I' in which case a float and/or integer array is made

    For 3-D arrays, the channel_axis argument tells which dimension of the
      array holds the channel data. 
    For 3-D arrays if one of the dimensions is 3, the mode is 'RGB'
      by default or 'YCbCr' if selected.  
    if the

    The Numeric array must be either 2 dimensional or 3 dimensional.
    """
    data = asarray(arr)
    if iscomplexobj(data):
        raise ValueError, "Cannot convert a complex-valued array."
    shape = list(data.shape)
    valid = len(shape)==2 or ((len(shape)==3) and \
                              ((3 in shape) or (4 in shape)))
    assert valid, "Not a suitable array shape for any mode."
    if len(shape) == 2:
        shape = (shape[1],shape[0]) # columns show up first
        if mode == 'F':
            image = Image.fromstring(mode,shape,data.astype('f').tostring())
            return image
        if mode in [None, 'L', 'P']:
            bytedata = bytescale(data,high=high,low=low,cmin=cmin,cmax=cmax)
            image = Image.fromstring('L',shape,bytedata.tostring())
            if pal is not None:
                image.putpalette(asarray(pal,typecode=_UInt8).tostring())
                # Becomes a mode='P' automagically.
            elif mode == 'P':  # default gray-scale
                pal = arange(0,256,1,typecode='b')[:,NewAxis] * \
                      ones((3,),typecode='b')[NewAxis,:]
                image.putpalette(asarray(pal,typecode=_UInt8).tostring())
            return image
        if mode == '1':  # high input gives threshold for 1
            bytedata = ((data > high)*255).astype('b')
            image = Image.fromstring('L',shape,bytedata.tostring())   
            image = image.convert(mode='1')
            return image
        if cmin is None:
            cmin = amin(ravel(data))
        if cmax is None:
            cmax = amax(ravel(data))
        data = (data*1.0 - cmin)*(high-low)/(cmax-cmin) + low
        if mode == 'I':
            image = Image.fromstring(mode,shape,data.astype('i').tostring())
        else:
            raise ValueError, _errstr
        return image

    # if here then 3-d array with a 3 or a 4 in the shape length.
    # Check for 3 in datacube shape --- 'RGB' or 'YCbCr'
    if channel_axis is None:
        if (3 in shape):
            ca = Numeric.nonzero(asarray(shape) == 3)[0]
        else:
            ca = Numeric.nonzero(asarray(shape) == 4)
            if len(ca):
                ca = ca[0]
            else:
                raise ValueError, "Could not find channel dimension."
    else:
        ca = channel_axis

    numch = shape[ca]
    if numch not in [3,4]:
        raise ValueError, "Channel axis dimension is not valid."

    bytedata = bytescale(data,high=high,low=low,cmin=cmin,cmax=cmax)
    if ca == 2:
        strdata = bytedata.tostring()
        shape = (shape[1],shape[0])
    elif ca == 1:
        strdata = transpose(bytedata,(0,2,1)).tostring()
        shape = (shape[2],shape[0])
    elif ca == 0:
        strdata = transpose(bytedata,(1,2,0)).tostring()
        shape = (shape[2],shape[1])
    if mode is None:
        if numch == 3: mode = 'RGB'
        else: mode = 'RGBA'


    if mode not in ['RGB','RGBA','YCbCr','CMYK']:
        raise ValueError, _errstr

    if mode in ['RGB', 'YCbCr']:
        assert numch == 3, "Invalid array shape for mode."
    if mode in ['RGBA', 'CMYK']:
        assert numch == 4, "Invalid array shape for mode."

    # Here we know data and mode is coorect
    image = Image.fromstring(mode, shape, strdata)
    return image

def imrotate(arr,angle,interp='bilinear'):
    """Rotate an image counter-clockwise by angle degrees.

    Interpolation methods can be:
        'nearest' :  for nearest neighbor
        'bilinear' : for bilinear
        'cubic' or 'bicubic' : for bicubic 
    """
    arr = asarray(arr)
    func = {'nearest':0,'bilinear':2,'bicubic':3,'cubic':3}
    im = toimage(arr)
    im = im.rotate(angle,resample=func[interp])
    return fromimage(im)

def imresize(arr,newsize,interp='bilinear',mode=None):
    newsize=list(newsize)
    newsize.reverse()
    newsize = tuple(newsize)
    arr = asarray(arr)
    func = {'nearest':0,'bilinear':2,'bicubic':3,'cubic':3}
    im = toimage(arr,mode=mode)
    im = im.resize(newsize,resample=func[interp])
    return fromimage(im)
    
def imshow(arr):
    """Simple showing of an image through an external viewer.
    """
    im = toimage(arr)
    if (len(arr.shape) == 3) and (arr.shape[2] == 4):
        try:
            import os
            im.save('/tmp/scipy_imshow.png')
            if os.system("(xv /tmp/scipy_imshow.png; rm -f /tmp/scipy_imshow.png)&"):
                raise RuntimeError
            return
        except:
            print "Warning: Alpha channel may not be handled correctly."
            
    im.show()
    return

def imresize(arr,size):
    """Resize an image.

    If size is an integer it is a percentage of current size.
    If size is a float it is a fraction of current size.
    If size is a tuple it is the size of the output image.
    """
    im = toimage(arr)
    ts = type(size)
    if ts is types.IntType:
        size = size / 100.0
    if type(size) is types.FloatType:
        size = (im.size[0]*size,im.size[1]*size)
    else:
        size = (size[1],size[0])
    imnew = im.resize(size)
    return fromimage(imnew)


def imfilter(arr,ftype):
    """Simple filtering of an image.

    type can be:
            'blur', 'contour', 'detail', 'edge_enhance', 'edge_enhance_more',
            'emboss', 'find_edges', 'smooth', 'smooth_more', 'sharpen'
    """
    _tdict = {'blur':ImageFilter.BLUR,
              'contour':ImageFilter.CONTOUR,
              'detail':ImageFilter.DETAIL,
              'edge_enhance':ImageFilter.EDGE_ENHANCE,
              'edge_enhance_more':ImageFilter.EDGE_ENHANCE_MORE,
              'emboss':ImageFilter.EMBOSS,
              'find_edges':ImageFilter.FIND_EDGES,
              'smooth':ImageFilter.SMOOTH,
              'smooth_more':ImageFilter.SMOOTH_MORE,
              'sharpen':ImageFilter.SHARPEN
              }

    im = toimage(arr)
    if ftype not in _tdict.keys():
        raise ValueError, "Unknown filter type."
    return fromimage(im.filter(_tdict[ftype]))
           
 
def radon(arr,theta=None):
    if theta is None:
        theta = mgrid[0:180]
    s = zeros((arr.shape[1],len(theta)),'d')
    k = 0
    for th in theta:
        im = imrotate(arr,-th)
        s[:,k] = sum(im,axis=0)
        k += 1
    return s
        



"""
SciPy --- A scientific computing package for Python
===================================================

Available subpackages
---------------------

"""
# Modules to import under the scipy namespace
from scipy_version import scipy_version as __version__
from scipy_base import *
from helpmod import *

from scipy_test.testing import ScipyTest

#-------- doc string hooks --------#

def _level_docs(module=None,_cache=[]):
    if module is None:
        return _cache
    try:
        title = module.__dict__['__doc_title__']
    except KeyError:
        try:
            title = module.__dict__['__doc__']
            title = title.lstrip().split('\n',1)[0]
        except KeyError:
            title = ' N/A'
    _cache.append((module.__name__,title))

def _pkg_titles():
    level_docs = _level_docs()
    lengths = [len(name) for (name,title) in level_docs]
    max_length = max(lengths)
    lines = []
    for (name,title) in level_docs:
        w = max_length - len(name)
        lines.append('%s%s --- %s' % (name, w*' ', title))
    return '\n'.join(lines)

#----- Import packages ----#

def _import_packages():
    """ Import packages in scipy directory that implement
    info_<packagename>.py. See DEVELOPERS.txt for more info.
    """
    from glob import glob
    import os
    frame = sys._getframe(1)

    for info_file in glob(os.path.join(__path__[0],'*','info_*.py')):

        package_name = os.path.basename(os.path.dirname(info_file))
        if package_name != os.path.splitext(os.path.basename(info_file))[0][5:]:
            print '  !! Mismatch of package name %r and %s' \
                  % (package_name, info_file)
            continue

        sys.path.insert(0,os.path.dirname(info_file))
        # TODO: catch exceptions here:
        exec 'import info_%s as info_module' % (package_name)
        del sys.path[0]
        
        if getattr(info_module,'ignore',0):
            continue

        global_symbols = getattr(info_module,'global_symbols',[])
        
        if getattr(info_module,'postpone_import',1):
            code = '%s = ppimport(%r)' % (package_name,package_name)

            for name in global_symbols:
                code += '\n%s = ppimport_attr(%s,%r)' % (name,package_name,name)
        else:
            code = 'import %s' % (package_name)
            # XXX: Should we check the existence of package.test? Warn?
            code += '\n%s.test = ScipyTest(%s).test' % (package_name,package_name)
            if global_symbols:
                code += '\nfrom '+package_name+' import '+','.join(global_symbols)
        # XXX: Should we catch exceptions here??
        exec (code, frame.f_globals,frame.f_locals)

        _level_docs(info_module)
        # XXX: Ugly hack to fix package name:
        code = '_level_docs()[-1] = (%s.__name__,_level_docs()[-1][1])' \
               % (package_name)
        exec (code, frame.f_globals,frame.f_locals)

_import_packages()
del _import_packages

#Packages installed standalone:
weave = ppimport('weave');             _level_docs(weave)
wxPython_thread = ppimport_attr(ppimport('gui_thread'),'wxPython_thread')

#----------------------------------#

from common import *
from pilutil import *

#---- testing ----#

test = ScipyTest('scipy').test

#----- update doc string -------#

__doc__ += _pkg_titles()



#import Tkinter
#from Tkinter import *
#from Scientific.TkWidgets.TkPlotCanvas import *
from Numeric import *
from scipy_base.fastumath import *
#from _Graphics import *
#import PIL.ImageTk,PIL.Image
#ImageTk = PIL.ImageTk
#Image = PIL.Image
import gist, types, string, os
import Mplot
import scipy.io as io


def read_act(filename):
    fid = io.fopen('%s.act'%filename)
    p = fid.fread(256*3,'byte')
    fid.close()
    p.shape = (256,3)
    return p

Palette = {}

def make_palettes():
    p = arange(0,256,1,typecode='b')[:,NewAxis] * \
        ones((3,),typecode='b')[NewAxis,:]
    Palette['gray'] = p

    p = arange(0,256,1,typecode='b')[:,NewAxis] * \
        array([0,0,1],typecode='b')[NewAxis,:]    
    Palette['blue'] = p

    p = arange(0,256,1,typecode='b')[:,NewAxis] * \
        array([1,0,0],typecode='b')[NewAxis,:]        
    Palette['red'] = p

    p = arange(0,256,1,typecode='b')[:,NewAxis] * \
        array([0,1,0],typecode='b')[NewAxis,:]        
    Palette['green'] = p

    p = zeros((256,3),'b')
    dp = (256-40)/128.0
    p[:128,2] = arange(40,256,dp,typecode='b')[::-1]
    p[128:,0] = arange(40,256,dp,typecode='b')
    Palette['wave'] = p

    Palette['Aaasmo'] = read_act('%s/Research/Library/Python/Graphics/Aaasmo'% os.environ['HOME'])
    Palette['awave'] = read_act('%s/Research/Library/Python/Graphics/awave'% os.environ['HOME'])

def _interpolate_colormap(p,num):
    assert(len(p.shape)==2)
    assert(p.shape[1]<=3)
    pf = p.astype('d')
    pfnew = signaltools.resample(pf,num,axis=0)
    return pfnew.astype('b')
    
def read_gist_palettes():
    import commands


make_palettes()
read_gist_palettes()

##def NumPyToTkImage(data,size):
##    data = ZeroOrderHold(data,size)
##    shape = tuple(array(data.shape)[::-1])
##    return ImageTk.PhotoImage(Image.fromstring("L",shape,data.tostring()))

def _makebytes(data,top,cmin,cmax):
    if data.typecode == UnsignedInt8:
        return data
    if cmin is None:
        if cmax is None:
            bytedata = gist.bytscl(data,top=255)
        else:
            bytedata = gist.bytscl(data,cmax=cmax,top=255)
    else:
        if cmax is None:
            bytedata = gist.bytscl(data,cmin=cmin,top=255)
        else:
            bytedata = gist.bytscl(data,cmin=cmin,cmax=cmax,top=255)
    return bytedata

def _makewavebytes(data,bot,top,cmin,cmax):
    if top is None:
        top = 255
    if bot is None:
        bot = 1
    mid = (top+1) / 2        # Zero will get mapped to here.
    if cmin is None:
        cmin = min(ravel(data))
    if cmax is None:
        cmax = max(ravel(data))

    bytelow = (mid-bot)/(0.0-cmin) * data + mid
    bytehigh = (top - mid)/(cmax-0.0) * data + mid
    bytedata = where(greater(data,0),bytehigh,bytelow).astype('b')
    
    return bytedata

    
##def array2image(data,expand=None,top=255,cmin=None,cmax=None,p=None):
##    """array2image(data,expand=None,cmin=None,cmax=None,p=None)"""
##    assert(len(data.shape) == 2)
##    shape = (data.shape[1],data.shape[0])
##    bytedata = _makebytes(data,top,cmin,cmax)
##    if expand is not None:
##        if type(expand) is types.IntType:
##            expand = (expand,expand)
##        assert(type(expand) in [types.TupleType, ArrayType, types.ListType])
##        assert(expand[0] > 0 and expand[1] > 0)
##        size = tuple(array(shape)*array(expand))
##        bytedata = ZeroOrderHold(bytedata,size)
##        shape = (size[1],size[0])
##    image = Image.fromstring("L",shape,bytedata.tostring())
##    if p is not None:
##        image.putpalette(asarray(p,typecode='b').tostring())
##    return image

def write_frames(data,filename,dim=0,expand=None,cmin=None,cmax=None,p=None):
    """filelist =
    write_frames(data,filename,dim=0,expand=None,cmin=None,cmax=None,p=None)
    """
    assert(dim < 3)
    assert(len(data.shape) == 3)
    slobj = [slice(None)]*3
    filelist = []
    #data = _makebytes(data,255,cmin,cmax)  # scale all frames the same
    for k in range(data.shape[dim]):
        slobj[dim] = k
        im = array2image(data[slobj],expand,cmin=cmin,cmax=cmax,p=p)
        file = '%s%03d.png'%(filename,k)
        im.save(file)
        filelist.append(file)
    return filelist


def make_movie(data,filename,dim=0,expand=None,cmin=None,cmax=None,p=None):
    """make_movie(data,filename,dim=0,expand=None,cmin=None,cmax=None,p=None)"""
    assert(dim < 3)
    assert(len(data.shape) == 3)
    slobj = [slice(None)]*3
    filelist = []
    data = _makebytes(data,255,cmin,cmax)  # scale all frames the same
    for k in range(data.shape[dim]):
        slobj[dim] = k
        im = array2image(data[slobj],expand,top=255,cmin=cmin,cmax=cmax,p=p)
        im2 = im.convert('RGB')
        file = '/tmp/%s%03d.tiff'%(filename,k)
        im2.save(file)
        filelist.append(file)
    os.system('convert %s %s.mpg' % (string.join(filelist," "),filename))
    os.system('rm %s' % string.join(filelist," "))


def make_wavemovie(data,filename,dim=0,expand=None,cmin=None,cmax=None,p=None):
    """make_wavemovie(data,filename,dim=0,expand=None,cmin=None,cmax=None,p=None)"""
    assert(dim < 3)
    assert(len(data.shape) == 3)
    slobj = [slice(None)]*3
    filelist = []
    data = _makewavebytes(data,0,255,cmin,cmax)  # scale all frames the same
    for k in range(data.shape[dim]):
        slobj[dim] = k
        im = array2image(data[slobj],expand,top=255,cmin=cmin,cmax=cmax,p=p)
        file = '/tmp/%s%03d.gif'%(filename,k)
        im.save(file)
        filelist.append(file)
    os.system('convert %s %s.gif' % (string.join(filelist," "),filename))
    os.system('rm %s' % string.join(filelist," "))















# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
from shapetest import *
from graftypes import *

class Animation2d :
   """
   anim = Animation2d ( <keyword arguments> ) will create a class which
   supplies the controls for an animation. For the most part, the
   keyword arguments are the names of functions which perform various
   parts of the calculations for the animation, under control of a
   Plotter object. Here the keywords are:
     initialize = <name of an initialization function>. This function
        should have one argument, the name of the instantiation 'anim',
        and when called should initialize any internal variables needed
        before beginning to comput the animation.
     calculations = <name or list of names of calculations of coordinates>
        this routine (or these routines) are called from within a loop
        in the Plotter(s) associated with anim. Each of the calculations
        routines should have 'anim' as the argument. They should compute
        the current values of anim.x and anim.y, the coordinates of the
        curve(s) in this step of the animation. They start with whatever
        was 'initialize'd, then in subsequent calls, use 'update'd values.
        If more than one calculation is specified, then a plot command
        will be issued after each one.
     update = <name of routine to update the variables used in calculations>.
        This updates (increments, decrements) variables used in calculating
        the frames. Like the other functions, has 'anim' as its sole
        argument.
   The following keywords are scalar values:
     animation = 0/1 (If 1, supplies a smoother animation for, e. g.,
        making a movie. default value 1.)
     nsteps = number of animation steps desired. Default: 100.
     color = <value> where <value> is an integer from 0 to 63
        representing an entry in a color chart, or a
        common color name like "red", "blue", "background", etc.
     In the interest of speed, other keywords relating to curve type,
     thickness, etc., are currently not allowed.
   """

   def type (self) :
      return Animation2dType

   _AnimationError = "AnimationError"

   def __init__ ( self, * kwds , ** keywords ) :
       if len (kwds) == 1 :
          keywords = kwds[0]
       self.line_type = "unspecified"
       if keywords.has_key ( "initialize" ) :
          self.initialize = keywords ["initialize"]
       else :
          raise self._AnimationError , \
             "Animation2d: keyword 'initialize' missing."
       if keywords.has_key ( "calculations" ) :
          self.calculations = keywords ["calculations"]
          if is_scalar (self.calculations) :
             self.calculations = [self.calculations]
       else :
          raise self._AnimationError , \
             "Animation2d: keyword 'calculations' missing."
       if keywords.has_key ( "update" ) :
          self.update = keywords ["update"]
       else :
          raise self._AnimationError , \
             "Animation2d: keyword 'update' missing."
       if keywords.has_key ( "animation" ) :
          self.animation = keywords ["animation"]
       else :
          self.animation = 1
       if keywords.has_key ( "nsteps" ) :
          self.nsteps = keywords ["nsteps"]
       else :
          self.nsteps = 100
       if keywords.has_key ( "color" ) :
          self.color = keywords ["color"]
       else :
          self.color = "fg"

   def new ( self, ** keywords ) :
       """ new (...keyword arguments...) allows you to reuse a
       previously existing animation.
       """
       del self.color, self.nsteps, self.animation, self.calculations, \
           self.initialize, self.update
       self.__init__ ( keywords )

   def set ( self , ** keywords ) :
       """ set (...keyword arguments...) allows you to set individual
       animation characteristics. No error checking is done.
       """
       for k in keywords.keys ():
           if k == "type" :
              self.line_type = keywords ["type"]
           else :
              setattr (self, k, keywords [k])



""" Machine limits for Float32 and Float64.
"""

__all__ = ['float_epsilon','float_tiny','float_min',
           'float_max','float_precision','float_resolution',
           'double_epsilon','double_tiny','double_min','double_max',
           'double_precision','double_resolution']

from machar import machar_double, machar_single

float_epsilon = machar_single.epsilon
float_tiny = machar_single.tiny
float_max = machar_single.huge
float_min = -float_max
float_precision = machar_single.precision
float_resolution = machar_single.resolution

double_epsilon = machar_double.epsilon
double_tiny = machar_double.tiny
double_max = machar_double.huge
double_min = -double_max
double_precision = machar_double.precision
double_resolution = machar_double.resolution

if __name__ == '__main__':
    print 'float epsilon:',float_epsilon
    print 'float tiny:',float_tiny
    print 'double epsilon:',double_epsilon
    print 'double tiny:',double_tiny




"""
Linear algebra routines
=======================

 Linear Algebra Basics:

   inv        --- Find the inverse of a square matrix
   solve      --- Solve a linear system of equations
   solve_banded --- Solve a linear system of equations with a banded matrix
   det        --- Find the determinant of a square matrix
   norm       --- matrix and vector norm
   lstsq      --- Solve linear least-squares problem
   pinv       --- Pseudo-inverse (Moore-Penrose) using lstsq
   pinv2      --- Pseudo-inverse using svd

 Eigenvalues and Decompositions:

   eig        --- Find the eigenvalues and vectors of a square matrix
   eigvals    --- Find the eigenvalues of a square matrix
   lu         --- LU decomposition of a matrix
   lu_factor  --- LU decomposition returning unordered matrix and pivots
   lu_solve   --- solve Ax=b using back substitution with output of lu_factor
   svd        --- Singular value decomposition of a matrix
   svdvals    --- Singular values of a matrix
   diagsvd    --- construct matrix of singular values from output of svd
   orth       --- construct orthonormal basis for range of A using svd
   cholesky   --- Cholesky decomposition of a matrix
   cho_factor --- Cholesky decomposition for use in solving linear system
   cho_solve  --- Solve previously factored linear system
   qr         --- QR decomposition of a matrix
   schur      --- Schur decomposition of a matrix
   rsf2csf    --- Real to complex schur form
   hessenberg --- Hessenberg form of a matrix

 matrix Functions:
 
   expm       --- matrix exponential using Pade approx.
   expm2      --- matrix exponential using Eigenvalue decomp.
   expm3      --- matrix exponential using Taylor-series expansion
   logm       --- matrix logarithm
   cosm       --- matrix cosine
   sinm       --- matrix sine
   tanm       --- matrix tangent
   coshm      --- matrix hyperbolic cosine
   sinhm      --- matrix hyperbolic sine
   tanhm      --- matrix hyperbolic tangent
   signm      --- matrix sign
   sqrtm      --- matrix square root
   funm       --- Evaluating an arbitrary matrix function.

 Iterative linear systems solutions
   
   cg         --- Conjugate gradient (symmetric systems only)
   cgs        --- Conjugate gradient squared
   qmr        --- Quasi-minimal residual
   gmres      --- Generalized minimal residual
   bicg       --- Bi-conjugate gradient
   bicgstab   --- Bi-conjugate gradient stabilized
 
"""

postpone_import = 1



import Numeric
from Numeric import *
from type_check import isscalar, asarray

__all__ = ['atleast_1d','atleast_2d','atleast_3d','vstack','hstack',
           'column_stack','dstack','array_split','split','hsplit',
           'vsplit','dsplit','squeeze','apply_over_axes','expand_dims',
           'apply_along_axis']

def apply_along_axis(func1d,axis,arr,*args):
    """ Execute func1d(arr[i],*args) where func1d takes 1-D arrays
        and arr is an N-d array.  i varies so as to apply the function
        along the given axis for each 1-d subarray in arr.
    """
    nd = Numeric.rank(arr)
    if axis < 0: axis += nd
    if (axis >= nd):
        raise ValueError, "axis must be less than the rank; "+\
              "axis=%d, rank=%d." % (axis,)
    ind = [0]*(nd-1)
    dims = Numeric.shape(arr)
    i = zeros(nd,'O')
    indlist = range(nd)
    indlist.remove(axis)
    i[axis] = slice(None,None)
    outshape = take(shape(arr),indlist)
    put(i,indlist,ind)
    res = func1d(arr[i],*args)
    #  if res is a number, then we have a smaller output array
    if isscalar(res):
        outarr = zeros(outshape,asarray(res).typecode())
        outarr[ind] = res
        Ntot = product(outshape)
        k = 1
        while k < Ntot:
            # increment the index
            ind[-1] += 1
            n = -1
            while (ind[n] >= outshape[n]) and (n > (1-nd)):
                ind[n-1] += 1
                ind[n] = 0
                n -= 1
            put(i,indlist,ind)
            res = func1d(arr[i],*args)
            outarr[ind] = res
            k += 1
        return outarr
    else:
        Ntot = product(outshape)
        holdshape = outshape
        outshape = list(shape(arr))
        outshape[axis] = len(res)
        outarr = zeros(outshape,asarray(res).typecode())
        outarr[i] = res
        k = 1
        while k < Ntot:
            # increment the index
            ind[-1] += 1
            n = -1
            while (ind[n] >= holdshape[n]) and (n > (1-nd)):
                ind[n-1] += 1
                ind[n] = 0
                n -= 1
            put(i,indlist,ind)
            res = func1d(arr[i],*args)
            outarr[i] = res
            k += 1
        return outarr
        
     
def apply_over_axes(func, a, axes):
    """Apply a function over multiple axes, keeping the same shape
    for the resulting array.
    """
    val = asarray(a)
    N = len(val.shape)
    if not type(axes) in SequenceType:
        axes = (axes,)
    for axis in axes:
        if axis < 0: axis = N + axis
        args = (val, axis)
        val = expand_dims(func(*args),axis)
    return val

def expand_dims(a, axis):
    """Expand the shape of a by including NewAxis before given axis.
    """
    a = asarray(a)
    shape = a.shape
    if axis < 0:
        axis = axis + len(shape) + 1
    a.shape = shape[:axis] + (1,) + shape[axis:]
    return a

def squeeze(a):
    "Returns a with any ones from the shape of a removed"
    a = asarray(a)
    b = asarray(a.shape)
    val = reshape (a, tuple (compress (not_equal (b, 1), b)))
    return val

def atleast_1d(*arys):
    """ Force a sequence of arrays to each be at least 1D.

         Description:
            Force an array to be at least 1D.  If an array is 0D, the 
            array is converted to a single row of values.  Otherwise,
            the array is unaltered.
         Arguments:
            *arys -- arrays to be converted to 1 or more dimensional array.
         Returns:
            input array converted to at least 1D array.
    """
    res = []
    for ary in arys:
        ary = asarray(ary)
        if len(ary.shape) == 0: 
            result = Numeric.array([ary[0]])
        else:
            result = ary
        res.append(result)
    if len(res) == 1:
        return res[0]
    else:
        return res

def atleast_2d(*arys):
    """ Force a sequence of arrays to each be at least 2D.

         Description:
            Force an array to each be at least 2D.  If the array
            is 0D or 1D, the array is converted to a single
            row of values.  Otherwise, the array is unaltered.
         Arguments:
            arys -- arrays to be converted to 2 or more dimensional array.
         Returns:
            input array converted to at least 2D array.
    """
    res = []
    for ary in arys:
        ary = asarray(ary)
        if len(ary.shape) == 0: 
            ary = Numeric.array([ary[0]])
        if len(ary.shape) == 1: 
            result = ary[NewAxis,:]
        else: 
            result = ary
        res.append(result)
    if len(res) == 1:
        return res[0]
    else:
        return res
        
def atleast_3d(*arys):
    """ Force a sequence of arrays to each be at least 3D.

         Description:
            Force an array each be at least 3D.  If the array is 0D or 1D, 
            the array is converted to a single 1xNx1 array of values where 
            N is the orginal length of the array. If the array is 2D, the 
            array is converted to a single MxNx1 array of values where MxN
            is the orginal shape of the array. Otherwise, the array is 
            unaltered.
         Arguments:
            arys -- arrays to be converted to 3 or more dimensional array.
         Returns:
            input array converted to at least 3D array.
    """
    res = []
    for ary in arys:
        ary = asarray(ary)
        if len(ary.shape) == 0:
            ary = Numeric.array([ary[0]])
        if len(ary.shape) == 1:
            result = ary[NewAxis,:,NewAxis]
        elif len(ary.shape) == 2:
            result = ary[:,:,NewAxis]
        else: 
            result = ary
        res.append(result)
    if len(res) == 1:
        return res[0]
    else:
        return res


def vstack(tup):
    """ Stack arrays in sequence vertically (row wise)

        Description:
            Take a sequence of arrays and stack them veritcally
            to make a single array.  All arrays in the sequence
            must have the same shape along all but the first axis. 
            vstack will rebuild arrays divided by vsplit.
        Arguments:
            tup -- sequence of arrays.  All arrays must have the same 
                   shape.
        Examples:
            >>> import scipy
            >>> a = array((1,2,3))
            >>> b = array((2,3,4))
            >>> scipy.vstack((a,b))
            array([[1, 2, 3],
                   [2, 3, 4]])
            >>> a = array([[1],[2],[3]])
            >>> b = array([[2],[3],[4]])
            >>> scipy.vstack((a,b))
            array([[1],
                   [2],
                   [3],
                   [2],
                   [3],
                   [4]])

    """
    return Numeric.concatenate(map(atleast_2d,tup),0)

def hstack(tup):
    """ Stack arrays in sequence horizontally (column wise)

        Description:
            Take a sequence of arrays and stack them horizontally
            to make a single array.  All arrays in the sequence
            must have the same shape along all but the second axis.
            hstack will rebuild arrays divided by hsplit.
        Arguments:
            tup -- sequence of arrays.  All arrays must have the same 
                   shape.
        Examples:
            >>> import scipy
            >>> a = array((1,2,3))
            >>> b = array((2,3,4))
            >>> scipy.hstack((a,b))
            array([1, 2, 3, 2, 3, 4])
            >>> a = array([[1],[2],[3]])
            >>> b = array([[2],[3],[4]])
            >>> scipy.hstack((a,b))
            array([[1, 2],
                   [2, 3],
                   [3, 4]])

    """
    return Numeric.concatenate(map(atleast_1d,tup),1)

def column_stack(tup):
    """ Stack 1D arrays as columns into a 2D array

        Description:
            Take a sequence of 1D arrays and stack them as columns
            to make a single 2D array.  All arrays in the sequence
            must have the same length.
        Arguments:
            tup -- sequence of 1D arrays.  All arrays must have the same 
                   length.
        Examples:
            >>> import scipy
            >>> a = array((1,2,3))
            >>> b = array((2,3,4))
            >>> scipy.vstack((a,b))
            array([[1, 2],
                   [2, 3],
                   [3, 4]])

    """
    arrays = map(Numeric.transpose,map(atleast_2d,tup))
    return Numeric.concatenate(arrays,1)
    
def dstack(tup):
    """ Stack arrays in sequence depth wise (along third dimension)

        Description:
            Take a sequence of arrays and stack them along the third axis.
            All arrays in the sequence must have the same shape along all 
            but the third axis.  This is a simple way to stack 2D arrays 
            (images) into a single 3D array for processing.
            dstack will rebuild arrays divided by dsplit.
        Arguments:
            tup -- sequence of arrays.  All arrays must have the same 
                   shape.
        Examples:
            >>> import scipy
            >>> a = array((1,2,3))
            >>> b = array((2,3,4))
            >>> scipy.dstack((a,b))
            array([       [[1, 2],
                    [2, 3],
                    [3, 4]]])
            >>> a = array([[1],[2],[3]])
            >>> b = array([[2],[3],[4]])
            >>> scipy.dstack((a,b))
            array([[        [1, 2]],
                   [        [2, 3]],
                   [        [3, 4]]])
    """
    return Numeric.concatenate(map(atleast_3d,tup),2)

def _replace_zero_by_x_arrays(sub_arys):
    for i in range(len(sub_arys)):
        if len(Numeric.shape(sub_arys[i])) == 0:
            sub_arys[i] = Numeric.array([])
        elif Numeric.sometrue(Numeric.equal(Numeric.shape(sub_arys[i]),0)):
            sub_arys[i] = Numeric.array([])   
    return sub_arys
    
def array_split(ary,indices_or_sections,axis = 0):
    """ Divide an array into a list of sub-arrays.

        Description:
           Divide ary into a list of sub-arrays along the
           specified axis.  If indices_or_sections is an integer,
           ary is divided into that many equally sized arrays.
           If it is impossible to make an equal split, each of the
           leading arrays in the list have one additional member.  If
           indices_or_sections is a list of sorted integers, its
           entries define the indexes where ary is split.

        Arguments:
           ary -- N-D array.
              Array to be divided into sub-arrays.
           indices_or_sections -- integer or 1D array.
              If integer, defines the number of (close to) equal sized
              sub-arrays.  If it is a 1D array of sorted indices, it
              defines the indexes at which ary is divided.  Any empty
              list results in a single sub-array equal to the original
              array.
           axis -- integer. default=0.
              Specifies the axis along which to split ary.
        Caveats:
           Currently, the default for axis is 0.  This
           means a 2D array is divided into multiple groups
           of rows.  This seems like the appropriate default, but
           we've agreed most other functions should default to
           axis=-1.  Perhaps we should use axis=-1 for consistency.
           However, we could also make the argument that SciPy
           works on "rows" by default.  sum() sums up rows of
           values.  split() will split data into rows.  Opinions?
    """
    try:
        Ntotal = ary.shape[axis]
    except AttributeError:
        Ntotal = len(ary)
    try: # handle scalar case.
        Nsections = len(indices_or_sections) + 1
        div_points = [0] + list(indices_or_sections) + [Ntotal]
    except TypeError: #indices_or_sections is a scalar, not an array.
        Nsections = int(indices_or_sections)
        if Nsections <= 0:
            raise ValueError, 'number sections must be larger than 0.'
        Neach_section,extras = divmod(Ntotal,Nsections)
        section_sizes = [0] + \
                        extras * [Neach_section+1] + \
                        (Nsections-extras) * [Neach_section]
        div_points = Numeric.add.accumulate(Numeric.array(section_sizes))

    sub_arys = []
    sary = Numeric.swapaxes(ary,axis,0)
    for i in range(Nsections):
        st = div_points[i]; end = div_points[i+1]
        sub_arys.append(Numeric.swapaxes(sary[st:end],axis,0))

    # there is a wierd issue with array slicing that allows
    # 0x10 arrays and other such things.  The following cluge is needed
    # to get around this issue.
    sub_arys = _replace_zero_by_x_arrays(sub_arys)
    # end cluge.

    return sub_arys

def split(ary,indices_or_sections,axis=0):
    """ Divide an array into a list of sub-arrays.

        Description:
           Divide ary into a list of sub-arrays along the
           specified axis.  If indices_or_sections is an integer,
           ary is divided into that many equally sized arrays.
           If it is impossible to make an equal split, an error is 
           raised.  This is the only way this function differs from
           the array_split() function. If indices_or_sections is a 
           list of sorted integers, its entries define the indexes
           where ary is split.

        Arguments:
           ary -- N-D array.
              Array to be divided into sub-arrays.
           indices_or_sections -- integer or 1D array.
              If integer, defines the number of (close to) equal sized
              sub-arrays.  If it is a 1D array of sorted indices, it
              defines the indexes at which ary is divided.  Any empty
              list results in a single sub-array equal to the original
              array.
           axis -- integer. default=0.
              Specifies the axis along which to split ary.
        Caveats:
           Currently, the default for axis is 0.  This
           means a 2D array is divided into multiple groups
           of rows.  This seems like the appropriate default, but
           we've agreed most other functions should default to
           axis=-1.  Perhaps we should use axis=-1 for consistency.
           However, we could also make the argument that SciPy
           works on "rows" by default.  sum() sums up rows of
           values.  split() will split data into rows.  Opinions?
    """
    try: len(indices_or_sections)
    except TypeError:
        sections = indices_or_sections
        N = ary.shape[axis]
        if N % sections:
            raise ValueError, 'array split does not result in an equal division'
    res = array_split(ary,indices_or_sections,axis)
    return res

def hsplit(ary,indices_or_sections):
    """ Split ary into multiple columns of sub-arrays

        Description:
            Split a single array into multiple sub arrays.  The array is
            divided into groups of columns.  If indices_or_sections is
            an integer, ary is divided into that many equally sized sub arrays.
            If it is impossible to make the sub-arrays equally sized, the
            operation throws a ValueError exception. See array_split and
            split for other options on indices_or_sections.                        
        Arguments:
           ary -- N-D array.
              Array to be divided into sub-arrays.
           indices_or_sections -- integer or 1D array.
              If integer, defines the number of (close to) equal sized
              sub-arrays.  If it is a 1D array of sorted indices, it
              defines the indexes at which ary is divided.  Any empty
              list results in a single sub-array equal to the original
              array.
        Returns:
            sequence of sub-arrays.  The returned arrays have the same 
            number of dimensions as the input array.
        Related:
            hstack, split, array_split, vsplit, dsplit.           
        Examples:
            >>> import scipy
            >>> a= array((1,2,3,4))
            >>> scipy.hsplit(a,2)
            [array([1, 2]), array([3, 4])]
            >>> a = array([[1,2,3,4],[1,2,3,4]])
            [array([[1, 2],
                   [1, 2]]), array([[3, 4],
                   [3, 4]])]
                   
    """
    if len(Numeric.shape(ary)) == 0:
        raise ValueError, 'hsplit only works on arrays of 1 or more dimensions'
    if len(ary.shape) > 1:
        return split(ary,indices_or_sections,1)
    else:
        return split(ary,indices_or_sections,0)
        
def vsplit(ary,indices_or_sections):
    """ Split ary into multiple rows of sub-arrays

        Description:
            Split a single array into multiple sub arrays.  The array is
            divided into groups of rows.  If indices_or_sections is
            an integer, ary is divided into that many equally sized sub arrays.
            If it is impossible to make the sub-arrays equally sized, the
            operation throws a ValueError exception. See array_split and
            split for other options on indices_or_sections.
        Arguments:
           ary -- N-D array.
              Array to be divided into sub-arrays.
           indices_or_sections -- integer or 1D array.
              If integer, defines the number of (close to) equal sized
              sub-arrays.  If it is a 1D array of sorted indices, it
              defines the indexes at which ary is divided.  Any empty
              list results in a single sub-array equal to the original
              array.
        Returns:
            sequence of sub-arrays.  The returned arrays have the same 
            number of dimensions as the input array.      
        Caveats:
           How should we handle 1D arrays here?  I am currently raising
           an error when I encounter them.  Any better approach?      
           
           Should we reduce the returned array to their minium dimensions
           by getting rid of any dimensions that are 1?
        Related:
            vstack, split, array_split, hsplit, dsplit.
        Examples:
            import scipy
            >>> a = array([[1,2,3,4],
            ...            [1,2,3,4]])
            >>> scipy.vsplit(a)
            [array([       [1, 2, 3, 4]]), array([       [1, 2, 3, 4]])]
                   
    """
    if len(Numeric.shape(ary)) < 2:
        raise ValueError, 'vsplit only works on arrays of 2 or more dimensions'
    return split(ary,indices_or_sections,0)

def dsplit(ary,indices_or_sections):
    """ Split ary into multiple sub-arrays along the 3rd axis (depth)

        Description:
            Split a single array into multiple sub arrays.  The array is
            divided into groups along the 3rd axis.  If indices_or_sections is
            an integer, ary is divided into that many equally sized sub arrays.
            If it is impossible to make the sub-arrays equally sized, the
            operation throws a ValueError exception. See array_split and
            split for other options on indices_or_sections.                        
        Arguments:
           ary -- N-D array.
              Array to be divided into sub-arrays.
           indices_or_sections -- integer or 1D array.
              If integer, defines the number of (close to) equal sized
              sub-arrays.  If it is a 1D array of sorted indices, it
              defines the indexes at which ary is divided.  Any empty
              list results in a single sub-array equal to the original
              array.
        Returns:
            sequence of sub-arrays.  The returned arrays have the same 
            number of dimensions as the input array.
        Caveats:
           See vsplit caveats.       
        Related:
            dstack, split, array_split, hsplit, vsplit.
        Examples:
            >>> a = array([[[1,2,3,4],[1,2,3,4]]])
            [array([       [[1, 2],
                    [1, 2]]]), array([       [[3, 4],
                    [3, 4]]])]
                                       
    """
    if len(Numeric.shape(ary)) < 3:
        raise ValueError, 'vsplit only works on arrays of 3 or more dimensions'
    return split(ary,indices_or_sections,2)




from ga_util import *
import UserList
import copy

class ga_list(UserList.UserList):
	def data_clone(self):
		new = shallow_clone(self)
		new.data = map(lambda x: x.clone(),self.data)
		return new		
	def touch(self): pass
	def __setslice__(self, i, j, list):
		if type(list) == type(self.data): self.data[i:j] = list
		else: self.data[i:j] = list.data
	def __getslice__(self, i, j):
		new = shallow_clone(self)
		new.data = self.data[i:j]
		new.touch()
		return new
	def __add__(self, list):
		new = shallow_clone(self)
		if type(list) == type(self.data): new.data = self.data + list			
		else: new.data = self.data + list.data			
		new.touch()
		return new
	def __radd__(self, list):
		new = shallow_clone(self)
		if type(list) == type(self.data): new.data = list + self.data		
		else:	new.data = list.data + self.data			
		new.touch()
		return new
	def __mul__(self, n):
		new = shallow_clone(self)
		new.data = self.data*n
		new.touch()
		return new
	__rmul__ = __mul__
	def __cmp__(self, other): return cmp(self.__dict__,other.__dict__)
	
class gene_list(ga_list):
	pass
	"""
	def __setitem__(self, i, item):
 		if(hasattr(item,'_value')): self.data[i] = item
 		else: self.data[i]._value = item
	def __setslice__(self, i, j, list):
		if type(list) == type(self.data):
			if(hasattr(list[0],'_value')): 
				self.data[i:j] = list
			else:
				for k in range(i,j): self.data[k]._value = list[i]	
		else:
			self.data[i:j] = list.data
	"""


""" Basic functions for manipulating 2d arrays

"""

__all__ = ['diag','eye','fliplr','flipud','rot90','bmat']

from Numeric import *
from type_check import asarray
import Matrix

def fliplr(m):
    """ returns a 2-D matrix m with the rows preserved and columns flipped 
        in the left/right direction.  Only works with 2-D arrays.
    """
    m = asarray(m)
    if len(m.shape) != 2:
        raise ValueError, "Input must be 2-D."
    return m[:, ::-1]

def flipud(m):
    """ returns a 2-D matrix with the columns preserved and rows flipped in
        the up/down direction.  Only works with 2-D arrays.
    """
    m = asarray(m)
    if len(m.shape) != 2:
        raise ValueError, "Input must be 2-D."
    return m[::-1]
    
# reshape(x, m, n) is not used, instead use reshape(x, (m, n))

def rot90(m, k=1):
    """ returns the matrix found by rotating m by k*90 degrees in the 
        counterclockwise direction.
    """
    m = asarray(m)
    if len(m.shape) != 2:
        raise ValueError, "Input must be 2-D."
    k = k % 4
    if k == 0: return m
    elif k == 1: return transpose(fliplr(m))
    elif k == 2: return fliplr(flipud(m))
    else: return fliplr(transpose(m))  # k==3
    
def eye(N, M=None, k=0, typecode='d'):
    """ eye returns a N-by-M matrix where the  k-th diagonal is all ones, 
        and everything else is zeros.
    """
    if M is None: M = N
    if type(M) == type('d'): 
        typecode = M
        M = N
    m = equal(subtract.outer(arange(N), arange(M)),-k)
    if typecode is None:
        return m
    else:
        return m.astype(typecode)

def diag(v, k=0):
    """ returns the k-th diagonal if v is a matrix or returns a matrix 
        with v as the k-th diagonal if v is a vector.
    """
    v = asarray(v)
    s = v.shape
    if len(s)==1:
        n = s[0]+abs(k)
        if k > 0:
            v = concatenate((zeros(k, v.typecode()),v))
        elif k < 0:
            v = concatenate((v,zeros(-k, v.typecode())))
        return eye(n, k=k)*v
    elif len(s)==2:
        v = add.reduce(eye(s[0], s[1], k=k)*v)
        if k > 0: return v[k:]
        elif k < 0: return v[:k]
        else: return v
    else:
            raise ValueError, "Input must be 1- or 2-D."


def _from_string(str,gdict,ldict):
    rows = str.split(';')
    rowtup = []
    for row in rows:
        trow = row.split(',')
        coltup = []
        for col in trow:
            col = col.strip()
            try:
                thismat = gdict[col]
            except KeyError:
                try:
                    thismat = ldict[col]
                except KeyError:
                    raise KeyError, "%s not found" % (col,)
                                    
            coltup.append(thismat)
        rowtup.append(concatenate(coltup,axis=-1))
    return concatenate(rowtup,axis=0)

import sys
def bmat(obj,gdict=None,ldict=None):
    """Build a matrix object from string, nested sequence, or array.

    Ex:  F = bmat('A, B; C, D')  
         F = bmat([[A,B],[C,D]])
         F = bmat(r_[c_[A,B],c_[C,D]])

        all produce the same Matrix Object    [ A  B ]
                                              [ C  D ]
                                      
        if A, B, C, and D are appropriately shaped 2-d arrays.
    """
    if isinstance(obj, types.StringType):
        if gdict is None:
            # get previous frame
            frame = sys._getframe().f_back
            glob_dict = frame.f_globals
            loc_dict = frame.f_locals
        else:
            glob_dict = gdict
            loc_dict = ldict
        
        return Matrix.Matrix(_from_string(obj, glob_dict, loc_dict))
    
    if isinstance(obj, (types.TupleType, types.ListType)):
        # [[A,B],[C,D]]
        arr_rows = []
        for row in obj:
            if isinstance(row, ArrayType):  # not 2-d
                return Matrix.Matrix(concatenate(obj,axis=-1))
            else:
                arr_rows.append(concatenate(row,axis=-1))
        return Matrix.Matrix(concatenate(arr_rows,axis=0))
    if isinstance(obj, ArrayType):
        return Matrix.Matrix(obj)




#
# weave - C/C++ integration
#

from info_weave import __doc__
from weave_version import weave_version as __version__

try:
    from blitz_tools import blitz
except ImportError:
    pass # Numeric wasn't available    
    
from inline_tools import inline
import ext_tools
from ext_tools import ext_module, ext_function
try:
    from accelerate_tools import accelerate
except:
    pass

from scipy_test.testing import ScipyTest
test = ScipyTest('weave').test



"""
    some basic language objects
    if you don't specify a node_type for these, it is usually a good idea to 
    not allow crossovers at these nodes.  That would allow two (maybe different)
    list_range nodes to exchange places.
"""

import tree
import gene


"""
import weakdict
class opt_object(tree.tree_node,weakdict.WeakValue):
	opt_dict = weakdict.WeakDict()
	def __init__(self,node_type, sub_nodes):
		tree.tree_node.__init__(self,sub_nodes,node_type=node_type)
		weakdict.WeakValue.__init__(self)
		opt_object.opt_dict[id(self)] = self
	def _create(self,parent = None):	
		new = tree.tree_node._create(self,parent)
		weakdict.WeakValue.__init__(new)
		self._WeakValue__cid = None #force a reset of the __cid value
		weakdict.WeakValue.__init__(new) #now get a new value for it
		opt_object.opt_dict[id(new)] = new
		return new		
	def __del__(self): 
		tree.tree_node.__del__(self)
		weakdict.WeakValue.__del__(self)
"""		

class opt_object(tree.tree_node):
	def __init__(self,node_type, sub_nodes):
		tree.tree_node.__init__(self,sub_nodes,node_type=node_type)

class float_range(gene.float_gene,opt_object):
	optimize = 1
	def __init__(self,bounds,node_type='float_range', sub_nodes = 0):
		opt_object.__init__(self,node_type,sub_nodes)
		gene.float_gene.__init__(self,bounds[:2])
		tree.tree_node.__init__(self,sub_nodes,node_type=node_type)
		if(len(bounds) == 3): self.default = bounds[2]
		else: self.default = (bounds[0] + bounds[1])/2.
		self._value = self.default
		print self._value
	def clone(self,parent = None):return tree.tree_node.clone(self,parent)			
	"""
	def mutate(self):
		m = gene.float_gene.mutate(self)
		if(m and self.parent): self.parent.recalc(force_parent=1)
		return m
	"""
	def scale(self,sc):
		self.bounds = (self.bounds[0]*sc,self.bounds[1]*sc)
		self.default = self.default*sc
		self._value = self._value*sc
	def defaultize(self):
		self._value = self.default
		for child in self._children: child.defaultize()		
	def create(self,parent):
		new = tree.tree_node.create(self,parent)
		new.initialize() 
		gene.float_gene.initialize(new) 		
		return new
	def __del__(self):
#		gene.float_gene.__del__(self)
		opt_object.__del__(self)	
	def __repr__(self):
		try: 
			val = self.value()
			if( val < .01 or val > 1000): v = "%4.3e" % self.value()
			else: v = "%4.3f" % self.value()				
		except gene.GAError: v = 'not initialized'
		self.label = '%s = %s' % (self.node_type, v)		
		return tree.tree_node.__repr__(self)
import math
class log_float_range(gene.log_float_gene,opt_object):
	optimize = 1
	def __init__(self,bounds,node_type='log_float_range', sub_nodes = 0):
		gene.log_float_gene.__init__(self,bounds[:2])
		opt_object.__init__(self,node_type,sub_nodes)
		if(len(bounds) == 3): self.default = bounds[2]
		else: self.default = (bounds[0] + bounds[1])/2.
		self._value = math.log10(self.default)
	def clone(self,parent = None):return tree.tree_node.clone(self,parent)		
	"""
	def mutate(self):
		m=gene.log_float_gene.mutate(self)
		if(m and self.parent): self.parent.recalc(force_parent=1)
		return m
	"""
	def scale(self,sc):
		self.default = self.default*sc
		sc = log10(sc)
		self.bounds = (self.bounds[0]*sc,self.bounds[1]*sc)
		self._value = self._value*sc
	def defaultize(self):
		self._value = self.default
		for child in self._children: child.defaultize()		
	def create(self,parent):
		new = tree.tree_node.create(self,parent)
		new.initialize() 
		gene.log_float_gene.initialize(new) 		
		return new
	def __del__(self):
#		gene.log_float_gene.__del__(self)
		opt_object.__del__(self)
	def __repr__(self):
		try: 
			val = self.value()
			if( val < .01 or val > 1000): v = "%4.3e" % self.value()
			else: v = "%4.3f" % self.value()				
		except gene.GAError: v = 'not initialized'
		self.label = '%s = %s' % (self.node_type, v)		
		return tree.tree_node.__repr__(self)
		
class list_range(gene.list_gene,opt_object):
	optimize = 1
	def __init__(self,allele_set,node_type='list_range', default=None, sub_nodes = 0):
		gene.list_gene.__init__(self,allele_set)
		opt_object.__init__(self,node_type,sub_nodes)
		gene.list_gene.initialize(self) # prevents trouble in tree generation
		if(default): self.default = default
		else: self.default = allele_set[int(len(allele_set)/2.)] #the center item
		self._value = self.default
	"""
	def mutate(self):
		m=gene.list_gene.mutate(self)
		if(m and self.parent): self.parent.recalc(force_parent=1)
		return m
	"""
	def clone(self,parent = None):return tree.tree_node.clone(self,parent)		
	def scale(self,sc):
		for i in range(len(self.allele_set)): 
			self.allele_set[i] = self.allele_set[i] *sc
		self.default = self.default*sc
		self._value = self._value*sc
	def defaultize(self):
		self._value = self.default
		for child in self._children: child.defaultize()		
	def create(self,parent):
		new = tree.tree_node.create(self,parent)
		new.initialize()
		gene.list_gene.initialize(new) 
		return new
	def __del__(self):
#		gene.list_gene.__del__(self)
		opt_object.__del__(self)
	def __repr__(self):
		self.label = '%s = %s' % (self.node_type, self.value())		
		return tree.tree_node.__repr__(self)

class val(gene.frozen,opt_object):
	optimize = 0
	def __init__(self,val,node_type='val',sub_nodes=0):
		gene.frozen.__init__(self,val)
		opt_object.__init__(self,node_type,sub_nodes)
	def clone(self,parent = None):return tree.tree_node.clone(self,parent)		
	def scale(self,sc): self._value = self._value*sc
	def defaultize(self): pass
	def create(self,parent):
		new = tree.tree_node.create(self,parent)
		new.initialize() 
		return new
	def __del__(self):
#		gene.frozen.__del__(self)
		opt_object.__del__(self)	
	def __repr__(self):
		self.label = '%s = %s' % (self.node_type, self.value())		
		return tree.tree_node.__repr__(self)


"""
These two routines are useful for picking off or replacing the nodes in a 
tree that should be that should be numerically optimized.  They are helpful if
your interested in using a gradient method to optimize some of the paramters
of the array
"""

def pick_numbers(node):
	start = []; lower = []; upper =[];
	for child in node.children():	
		s, l, u = pick_numbers(child)
		start = start + s
		lower = lower + l
		upper = upper + u
	#for now only works with float_genes	
	if hasattr(node,'optimize') and node.optimize == 1: 	
		s = node._value
		l,u = node.bounds
		start = start + [s]
		lower = lower + [l]
		upper = upper + [u]
	else:
		print 'no opt:', node.__class__		
	return start, lower, upper

def put_numbers(node,vals, index = 0):
	for child in node.children():	
		index = put_numbers(child,vals,index)
	if hasattr(node,'optimize') and node.optimize == 1: 	
		s = node._value = vals[index]
		index = index + 1
	return index

"""
Grab the numerical nodes that need to be optimized so that you can directly
manipulate them
"""
def pick_optimize_nodes(node):
	nodes = [];
	for child in node.children():	
		nodes = nodes + pick_optimize_nodes(child)
	#for now only works with float_genes	
	if hasattr(node,'optimize') and node.optimize == 1: 	
		nodes = nodes + [node]
	return nodes